::                                                       v0.1
::                         Mazing.cmd
:: __________________________________________________________
:: A maze program written in native WinNT batch script that
:: includes several maze generation and solving algorithms as
:: well as a small plethora of options for console display,
:: stack size, stack orientation, node selection/direction
:: bias, entrance/exit points, color selection, and wall/box
:: characters, each with an automatic randomizer, and all of
:: it easily accessible through an animated custom menu.
:: Mazes are limited to a maximum practical string length of
:: 8186 characters. The maze dimensions will be automatically
:: increased/decreased if minimum/maximum size is exceeded.
:: 
:: Mazing operates using 100% WinNT batch script, but it will
:: use either BG.EXE or CursorPos.exe to place the cursor at
:: the upper-left if they are located in the system path.
:: 
::               Command Line / Mazing.ini File
:: __________________________________________________________
:: In addition to the menu, other User Variables are passed
:: by using either the Mazing.ini file or on the command line
:: by using the following syntax:
:: Mazing.cmd ["variable=value"] [variable:value] [...]
:: 
:: Several options for minimum/maximum columns/rows, lists
:: of characters for random walls/crumbs/colors, keys used by
:: the menu, and output logfile are accessed by command line.
:: A list of available user variables may be found under the
:: :mazing_init label located at the bottom of the script.
::
::                        Menu Options
:: __________________________________________________________
:: Maze Generators: perfect mazes, one path to any two points
::   Backtracker  - created by random depth-first search
::   Hunt & Kill  - like Backtracker, but more random
::   Growing Tree - certainly the most versatile generator
::   Iterative Division - 'recursive' division in a FOR loop
::   Eller's Algorithm  - simplest and fastest of them all
:: Maze solvers: always find the 1 solution to perfect mazes
::   Don't Solve - only generate and convert mazes
::   Wall Follow - left or right hand on wall, depth-first
::   Dead Filler - remove dead-ends until path is revealed
::   Path Finder - flood maze until exit found, retrace path
:: Entrance/Exit: where to place the maze entrance and exit
::   Inside - entrance=any random point, exit=farthest point
::   Border - entrance=point on border, exit=farthest border
::   Corner - entrance=random corner, exit=farthest corner
:: Width: # of vertical columns, limited by minCols/maxCols
:: Height: # of horizontal rows, limited by minRows/maxRows
:: Ancillary Setting: different for each maze generator
::   Hunt & Kill  - type of stack, 0=FIFO, <>0=LIFO
::   Growing Tree - list size, 0=all+LIFO, <0=FIFO, >0=LIFO
::   Division - wall open, 0=NW,1=SE,2=NW/SE,3=Mid,4+=Random
:: Node Selection Bias: % chance of choosing random or stack
::   Hunt & Kill   - 0=random from stack,100=next from stack
::   Growing Tree  - 0=random from list, 100=next from stack
::   Itr. Division - stack order, 0-7=pre-sets, 8+=random
:: Node Direction Bias: % chance of choosing each direction
::   0=most vertical, 50=even chance, 100=most horizontal
::
::                        Menu Options
:: __________________________________________________________
:: Convert: alter maze characters and colors
::   Random Wall   - replace wall character from random list
::   Random Box    - replace walls with 1 of 12 pre-set boxes
::   Don't Convert - continue without altering settings
::   Random - chose randomly from the previous three options
:: Delay: # of centiseconds to delay each iteration of the
::   generation/solving loop, to better view the algorithm
:: Wall Character:  wall used for initial maze generation
:: Crumb Character: trail used for initial maze generation
::   These two may be altered by Convert if it's active
:: Start Character:  used to indicate entrance to maze
:: Finish Character: used to indicate exit from maze
::   Never use the same characters for these four settings^!
:: Background Color: console display, random=078F
:: Foreground color: console display, random=1234569ABCDE
::   Random      - select randomly from each color list
::   Very Random - select randomly from all 16 colors
::   Colors are displayed immediately to aid in selection
:: Display Type: how to display maze in the console window
::   No Display - only window title and logFile are active
::   16 Color - change window size+color, show maze animation
::   BG Color - display with additional colors, requires
::              Carlos' BG.EXE v3.9se automatically generated
::              in %TEMP% if not found in system path.
:: 
:: 
:: project by CirothUngol                  September 27, 2018
::

@ECHO OFF
IF "%~1"=="grabKey" GOTO :grabKey

:: initialize script, start main loop
SETLOCAL EnableExtensions EnableDelayedExpansion
CALL :mazing_init %*
CALL :mazing_loop
ENDLOCAL
EXIT /B 0

:mazing_menu
:: replace random box characters
SET /A bx=%RANDOM%%%8,mPos=orgPos+mWide*mCh,pages=3
SET "mMsg5=!mMsg5[%mzOp0%]!"
SET "tMenu=!mzMenu!"
FOR /L %%y IN (0,1,5) DO FOR %%z IN ("!mBox_:~%%y,1!=!mBox%bx%:~%%y,1!") DO SET "tMenu=!tMenu:%%~z!"
FOR /L %%y IN (0,1,%opCnt%) DO FOR %%z IN (":%%y:=!mView%%y!") DO SET "tMenu=!tMenu:%%~z!"
SET "tMenu=!tMenu::A:=%menuKeys:~0,2%!"
SET "tMenu=!tMenu::B:=%menuKeys:~2,2%!"
SET "tMenu=!tMenu::C:=%menuKeys:~4,3%!"
SET "tMenu=!tMenu::D:=%menuKeys:~7,1%!"
SET "tMenu=!tMenu:~0,%mzOp5P1%!!mzOp5[%mzOp0%]!!tMenu:~%mzOp5P2%!"
SET "tMenu=!tMenu:~0,%mzOp6P1%!!mzOp6[%mzOp0%]!!tMenu:~%mzOp6P2%!"
IF !mzOp8! GEQ 1 IF !mzOp8! LEQ 12 SET "tMenu=!tMenu:~0,%mzOp8P1%!!box%mzOp8%!!tMenu:~%mzOp8P2%!"
SET "tMenu=!tMenu:~0,%msgP1%!!mMsg%mCh%!!tMenu:~%msgP2%!"
:: copy color settings, change mode, start menu
SET mbClr=!bgClr!& SET mfClr=!fgClr!
MODE CON COLS=%mWide% LINES=%mHigh%
:mazing_menu_display
TITLE Maze#!mazeCnt!: !title0! at %TIME: =0% on %DATE%
SET /A mrkPos=mPos-3
COLOR !mbClr!!mfClr!
!clear!
ECHO(
ECHO(!tMenu:~0,%mrkPos%!!mrk!!tMenu:~%mPos%!
:mazing_menu_loop
SET "mKey=" & SET /A mOp=-1,mt0=0,mt1=mWide-1
FOR /F "skip=1 delims=" %%# IN ('replace ? . /u /w') DO IF NOT DEFINED mKey SET "mKey=%%#"
:: mOp is 0=up, 1=down, 2=left, 3=right, 4=continue&resume, 5=quit&restart, 6=abort&exit, 7=display help
FOR /L %%A IN (0,1,7) DO IF /I "!mKey!"=="!menuKeys:~%%A,1!" SET mOp=%%A
IF !mOp! LSS 0 GOTO :mazing_menu_loop
IF !mOp! GEQ 4 GOTO :mazing_menu_exit
IF !mOp! EQU 0 SET /A mCh-=1 & IF !mCh! LSS 0 SET mCh=!opCnt!
IF !mOp! EQU 1 SET /A mCh+=1 & IF !mCh! GTR !opCnt! SET mCh=0
IF !mOp! LEQ 1 ( SET /A mPos=orgPos+mWide*mCh
	SET "tMenu=!tMenu:~0,%msgP1%!!mMsg%mCh%!!tMenu:~%msgP2%!"
	GOTO :mazing_menu_display)
IF !mOp! EQU 2 SET /A mzOp%mCh%-=1 & IF !mzOp%mCh%! LSS -1 SET /A mzOp%mCh%=opMax%mCh%
IF !mOp! EQU 3 SET /A mzOp%mCh%+=1 & IF !mzOp%mCh%! GTR !opMax%mCh%! SET mzOp%mCh%=-1
:mazing_menu_cfg
SET /A mt0=mt1=mzOp%mCh%,mt2=mPos+opSize
IF !mt0! EQU -1 SET mt1=Random
IF !mCh! EQU 0 (SET "tMenu=!tMenu:~0,%mzOp5P1%!!mzOp5[%mt0%]!!tMenu:~%mzOp5P2%!"
				SET "tMenu=!tMenu:~0,%mzOp6P1%!!mzOp6[%mt0%]!!tMenu:~%mzOp6P2%!"
				SET "mMsg5=!mMsg5[%mt0%]!"
				SET "mMsg6=!mMsg6[%mt0%]!"
				SET "mt1=!mzOp0[%mt0%]!")
IF !mCh! EQU 1  SET "mt1=!mzOp1[%mt0%]!"
IF !mCh! EQU 2  SET "mt1=!mzOp2[%mt0%]!" 'display string instead of number
IF !mCh! EQU 3  IF !mt0! EQU 0 (SET /A mt1=mzOp3=minCols
				) ELSE IF !mt0! LSS !minCols! SET /A mt1=mzOp3=-1 & SET mt1=Random
IF !mCh! EQU 4  IF !mt0! EQU 0 (SET /A mt1=mzOp4=minRows
				) ELSE IF !mt0! LSS !minRows! SET /A mt1=mzOp4=-1 & SET mt1=Random
IF !mCh! EQU 5  IF !mt0! GEQ 0 SET /A mt1=mt0-12
IF !mCh! EQU 6  IF !mt0! EQU 101 SET mt1=Newest/Oldest
IF !mCh! EQU 8 (SET "tMenu=!tMenu:~0,%mzOp8P1%!               !tMenu:~%mzOp8P2%!"
				IF !mzOp8! EQU 0 (SET mt1=Don't Convert
				) ELSE IF !mzOp8! EQU 13 (SET mt1=Random Box
				) ELSE IF !mzOp8! EQU 14 (SET mt1=Random Wall
				) ELSE IF !mzOp8! GTR 0  (SET mt1=Box#!mt0!
					SET "tMenu=!tMenu:~0,%mzOp8P1%!!box%mzOp8%!!tMenu:~%mzOp8P2%!"))
IF !mCh! EQU 9 (IF !mt0! LSS 0  SET /A mt1=mzOp9=opMax9-1
				IF !mt0! EQU !opMax9! SET /A mt0=mzOp9=0
				IF !mt0! EQU 0 SET mt1=None)
IF !mCh! GEQ 10 IF !mCh! LEQ 13 IF !mt0! GEQ 0 SET mt1=!chars:~%mt0%,1!
IF !mCh! EQU 14 (SET bColors=078F& SET bColorCnt=4
				IF !mt0! LSS 0 (SET mbClr=%bgClr%
				) ELSE IF !mt0! LEQ 15 (SET mt1=!hex:~%mt0%,1!& SET mbClr=!mt1!
				) ELSE SET mbClr=%bgClr%& SET mt1=Very Random& SET bColors=!hex!& SET bColorCnt=16)
IF !mCh! EQU 15 (SET fColors=1234569ABCDE& SET fColorCnt=12
				IF !mt0! LSS 0 (SET mfClr=%fgClr%
				) ELSE IF !mt0! LEQ 15 (SET mt1=!hex:~%mt0%,1!& SET mfClr=!mt1!
				) ELSE SET mfClr=%fgClr%& SET mt1=Very Random& SET fColors=!hex!& SET fColorCnt=16)
IF !mCh! EQU 16 (SET display=!mzOp16!& SET mt1=No Display
				IF !mzOp16! LSS 0 SET mt1=16 Color
				IF !mzOp16! GTR 0 SET mt1=BG Color)
SET "mt1=!mt1!             "
SET "mView!mCh!=!mt1:~0,%opSize%!"
IF "%~1" NEQ "" EXIT /B 0
SET "tMenu=!tMenu:~0,%mPos%!!mView%mCh%!!tMenu:~%mt2%!"
GOTO :mazing_menu_display
:mazing_menu_exit
:: 4=continue&resume, 5=quit&restart, 6=abort&exit, 7=display help
IF !mOp! EQU 6 EXIT /B 3%=   'check for script abort =%
IF !mOp! EQU 7 FOR /F "usebackq tokens=* delims=:" %%# IN ("%~f0") DO (
	IF "%%#"=="" (SET/A mt0+=1
		TITLE Mazing Help: page !mt0! of !pages!   %TIME: =0% %DATE%
		ECHO(
		%EKO%...press any key to continue
		PAUSE>NUL
		ECHO(
		ECHO(
		IF !mt0! EQU !pages! (SET/A mh1=mSize-mWide*2-1
			FOR /L %%: IN (0,!mWide!,!mh1!) DO (ECHO(!tMenu:~%%:,%mt1%!
				CALL :wait 3)
			GOTO :mazing_menu_display)
	) ELSE (ECHO(%%#
		CALL :wait 3)
)
SET "tMenu="
REM.>"%keyFile%"
START "" /B CMD /C ^""%~f0" grabKey %exitKey% "%keyFile%" 2^>NUL ^>NUL^"
SET bgClr=!mbClr!& SET fgClr=!mfClr!& SET delay=!mzOp9!
IF !display! EQU 0 EXIT /B 0
IF !display! GTR 0 ( BG.EXE >NUL 2>&1
	IF !ERRORLEVEL! EQU 9009 CALL :makeBG
	SET clear=BG.EXE Locate 0 0
	BG.EXE Cursor 0)
IF !mOp! EQU 5 EXIT /B 2
IF NOT DEFINED wide EXIT /B 0
MODE CON COLS=%wide% LINES=%modeHigh%
ECHO(!mz!
IF "!title0:~0,5!" NEQ "Eller" %EKO%!msg:~1!
IF !display! GTR 0 CALL :mazing_BGcolor
EXIT /B 0

:mazing_BGcolor
:: use BG.EXE to set unique colors for entrance, exit, labelTop, labelBtm, and message.
SET /A "xr1=bgnPos/wide, xc1=bgnPos-xr1*wide, xr2=endPos/wide, xc2=endPos-xr2*wide, xr3=rows*2, xr4=high+1"
SET /A "exClr=!RANDOM!%%(colorCnt-2), pClr=!RANDOM!%%(colorCnt-3), gClr=!RANDOM!%%(colorCnt-4), ex2Clr=!RANDOM!%%(colorCnt-5)"
SET "tClr=!colors:%bgClr%=!"
SET "tClr=!tClr:%fgClr%=!"
SET "exClr=!tClr:~%exClr%,1!"
SET "tClr=!tClr:%exClr%=!"
SET "pClr=!tClr:~%pClr%,1!"
SET "tClr=!tClr:%pClr%=!"
SET "gClr=!tClr:~%gClr%,1!"
SET "tClr=!tClr:%gClr%=!"
SET "ex2Clr=!tClr:~%ex2Clr%,1!"
IF !bgClr! LSS 7 SET "lbClr=F"
IF !fgClr! EQU 0 IF !bgClr! NEQ F SET "lbClr=F"
BG.EXE FCPrint !xr1! !xc1! !bgClr!!pClr! "!player!"
BG.EXE FCPrint 0 2 !bgClr!!lbClr! "!labelTop!"
BG.EXE FCPrint !xr4! 0 !bgClr!!lbClr! "!msg:~1!"
IF NOT DEFINED labelBtm EXIT /B 0
BG.EXE FCPrint !xr2! !xc2! !bgClr!!gClr! "!goal!"
BG.EXE FCPrint !xr3! 2 !bgClr!!lbClr! "!labelBtm!"
IF NOT DEFINED dfMode EXIT /B 0
IF !dfMode! LSS 0 ( SET "dfClr=!bgClr!!exClr!"
) ELSE SET "dfClr=!exClr!!exClr!"
EXIT /B 0

:mazing_loop
SETLOCAL
SET "startTime=%TIME% %DATE%"

:: choose random valid colors
IF !mzOp14! EQU 16 SET "bColors=!hex!"& SET "bColorCnt=16"
IF !mzOp15! EQU 16 SET "fColors=!hex!"& SET "fColorCnt=16"
SET /A "p1=!RANDOM!%%bColorCnt, p2=!RANDOM!%%fColorCnt"
SET "bgClr=!bColors:~%p1%,1!"
SET "fgClr=!fColors:~%p2%,1!"
IF "!bgClr!" EQU "!fgClr!" SET "fgClr=5"
FOR %%A IN (!badClrs!) DO IF "!bgClr!!fgClr!" EQU "%%A" (
	SET "bgClr=!goodClr:~0,1!" & SET "fgClr=!goodClr:~-1!")

:: select random maze characters
SET /A "p1=!RANDOM!%%wallCnt, p2=!RANDOM!%%hallCnt, p3=!RANDOM!%%crumbCnt, p4=!RANDOM!%%playerCnt, p5=!RANDOM!%%goalCnt"
SET "wall=!walls:~%p1%,1!"
SET "hall=!halls:~%p2%,1!"
SET "crumb=!crumbs:~%p3%,1!"
SET "player=!players:~%p4%,1!"
SET "goal=!goals:~%p5%,1!"
SET "xWall=!wall!"
IF "!xWall!"=="" SET xWall=#

:: random maze creation settings
:: mzSelect=1-4, rsBias=random/stack=0-199, hvBias=horz/vert=0-149, endPos=0=farthest point,1=far point on border,2+=far corner
SET /A "cols=!RANDOM!%%(maxCols-minCols+1)+minCols, rows=!RANDOM!%%(maxRows-minRows+1)+minRows"
SET /A "mzSelect=!RANDOM!%%4+1,rsBias=!RANDOM!%%200,hvBias=!RANDOM!%%300"
SET /A "hk1=!RANDOM!%%3,gt1=!RANDOM!%%36-12,id1=!RANDOM!%%16,id2=!RANDOM!%%16"
SET /A "rebuild=!RANDOM!%%16-3,svSelect=!RANDOM!%%3+1"
IF !hvBias! GEQ 250 ( SET /A hvBias-=225%=        '1 in 6  set hvBias=25 to 75 mid-range =%
) ELSE IF !hvBias! GEQ 200 ( SET /A hvBias-=175%= '1 in 6  set hvBias=25 to 75 mid-range =%
) ELSE IF !hvBias! GEQ 150 ( SET /A hvBias-=125%= '1 in 6  set hvBias=25 to 75 mid-range =%
) ELSE IF !hvBias! GEQ 100 SET /A hvBias-=75%=    '1 in 6  set hvBias=25 to 75 mid-range =%
IF !rsBias! GTR 174 ( SET /A rsBias=101%=         '1 in 8  set stack to alternate top/bottom =%
) ELSE IF !rsBias! GTR 149 ( SET /A rsBias=0%=    '1 in 8  set stack always random =%
) ELSE IF !rsBias! GTR 99 SET /A rsBias-=75%=     '1 in 4  set rsBias=25 to 75 mid-range =%
IF !gt1! GTR 12 SET "gt1=0"%=                     '1 in 3  set to list=all =%
IF !id1! EQU 0 SET "mzSelect=0"%=                 '1 in 16 set to Backtracker =%
IF !gt1:~-1! EQU 1 SET /A gt1+=1%=                'assure gtList NEQ 1 =%

:: check for user/menu settings
IF NOT EXIST "%keyFile%" CALL :mazing_menu
IF ERRORLEVEL 3 GOTO :mazing_stop
IF !mzOp0!  GEQ 0 SET "mzSelect=!mzOp0!"
IF !mzOp3!  GEQ 0 SET "cols=!mzOp3!"
IF !mzOp4!  GEQ 0 SET "rows=!mzOp4!"
IF !mzOp5!  GEQ 0 SET /A "hk1=gt1=id1=mzOp5-12"
IF !mzOp6!  GEQ 0 SET "rsBias=id2=!mzOp6!"
IF !mzOp7!  GEQ 0 SET "hvBias=!mzOp7!"
IF !mzOp9!  GEQ 0 SET "delay=!mzOp9!"
IF !mzOp10! GEQ 0 SET "wall=!chars:~%mzOp10%,1!"
IF !mzOp11! GEQ 0 SET "crumb=!chars:~%mzOp11%,1!"
IF !mzOp12! GEQ 0 SET "player=!chars:~%mzOp12%,1!"
IF !mzOp13! GEQ 0 SET "goal=!chars:~%mzOp13%,1!"
IF !mzOp14! GEQ 0 IF !mzOp14! LEQ 15 SET "bgClr=!hex:~%mzOp14%,1!"
IF !mzOp15! GEQ 0 IF !mzOp15! LEQ 15 SET "fgClr=!hex:~%mzOp15%,1!"
SET /A display=mzOp16,mt0=-2
IF !id1! LSS 0 SET /A id1*=-1
FOR %%A IN (!RANDOM! 0 1 2) DO FOR %%B IN (!RANDOM! 0 1 2) DO ( SET /A mt0+=1
	IF !mt0! EQU !mzOp2! SET /A "bgnPos=%%B%%3,endPos=%%A%%3")

:: check for valid maze size
SET /A "size=(cols*2+1)*(rows*2+1)"
IF !size! LSS !minSize! FOR /L %%A IN (1,1,50) DO (
	IF !size! LSS !minSize! SET /A "cols+=1,size=(cols*2+1)*(rows*2+1)"
	IF !size! LSS !minSize! SET /A "rows+=1,size=(cols*2+1)*(rows*2+1)")
IF !size! GTR !maxSize! FOR /L %%A IN (1,1,100) DO (
	IF !size! GTR !maxSize! SET /A "rows-=1,size=(cols*2+1)*(rows*2+1)"
	IF !size! GTR !maxSize! SET /A "cols-=1,size=(cols*2+1)*(rows*2+1)")
SET /A wide=cols*2+1,high=rows*2+1,size=wide*high,modeHigh=high+2

:: random maze entrance
CALL :mazing_bgnPos %bgnPos%

:: resize console, set COLOR
IF !display! NEQ 0 (
	MODE CON COLS=%wide% LINES=%modeHigh%
	COLOR !bgClr!!fgClr!)

:: create maze
IF !mzSelect! LEQ 0 CALL :mazing_hunt_kill 1 100 !hvBias!
IF !mzSelect! EQU 1 CALL :mazing_hunt_kill !hk1! !rsBias! !hvBias!
IF !mzSelect! EQU 2 CALL :mazing_growing_tree !gt1! !rsBias! !hvBias!
IF !mzSelect! EQU 3 CALL :mazing_itr_division !id1! !id2! !hvBias!
IF !mzSelect! GEQ 4 CALL :mazing_ellers !hvBias!
IF ERRORLEVEL 3 GOTO :mazing_stop
IF ERRORLEVEL 2 GOTO :mazing_restart
SET /A mazeCnt+=1
IF !display! GTR 0 ( BG.EXE Locate 1 0
	ECHO(!mz:~%wide%!
)
IF !display! LSS 0 ( !clear!
	ECHO(!mz!
	%EKO%!msg:~1!
)

:: find goal/exit position
CALL :mazing_endPos %endPos%
IF ERRORLEVEL 3 GOTO :mazing_stop
IF ERRORLEVEL 2 GOTO :mazing_restart

:: rebuild maze with box characters
IF !mzOp8! GEQ 0 SET "rebuild=!mzOp8!"
IF !mzOp8! EQU 13 SET /A "rebuild=!RANDOM!%%12+1"
IF !mzOp8! EQU 14 SET "rebuild=-1"
IF %rebuild% NEQ 0 CALL :mazing_rebuild !rebuild!
IF ERRORLEVEL 3 GOTO :mazing_stop
IF ERRORLEVEL 2 GOTO :mazing_restart
IF !display! NEQ 0 ( !clear!
	ECHO(!mz!
	%EKO%!msg:~1!
)
CALL :timeSince %startTime%

:: write maze and info to logfile
IF DEFINED logFile (
	SET "bz=!mz:=%xWall%!"
	FOR /L %%A IN (0,!wide!,!size!) DO IF %%A LSS !size! ECHO(!bz:~%%A,%wide%!>>"!logFile!"
	ECHO(!mm!>>"!logFile!"
	ECHO(created in !TS_! at !TIME: =0! on !DATE!>>"!logFile!"
	SET bz=
)

:: solve maze
IF !mzOp1! GEQ 0 SET "svSelect=!mzOp1!"
IF !svSelect! GTR 0 ( SET "startTime=%TIME% %DATE%"
	IF !svSelect! EQU 1 CALL :mazing_wall_follow !rebuild!
	IF !svSelect! EQU 2 CALL :mazing_dead_filler !rebuild!
	IF !svSelect! GEQ 3 CALL :mazing_path_finder !rebuild!
	IF ERRORLEVEL 3 GOTO :mazing_stop
	IF ERRORLEVEL 2 GOTO :mazing_restart
	IF !display! NEQ 0 ( !clear!
		ECHO(!mz!
		%EKO%!msg:~1!)
	CALL :timeSince !startTime!
	REM write solution and info to logfile
	IF DEFINED logFile (
		SET "mz=!mz:=%xWall%!"
		ECHO(>>"!logFile!"
		FOR /L %%A IN (0,!wide!,!size!) DO IF %%A LSS !size! ECHO(!mz:~%%A,%wide%!>>"!logFile!"
		ECHO(!mm!>>"!logFile!"
		ECHO(solved in !TS_! at !TIME: =0! on !DATE!>>"!logFile!"
	)
)
CALL :timeSince !totalTime!
	   IF !mazeCnt:~-2! EQU 11 ( SET "mm=!mazeCnt!th"
) ELSE IF !mazeCnt:~-2! EQU 12 ( SET "mm=!mazeCnt!th"
) ELSE IF !mazeCnt:~-2! EQU 13 ( SET "mm=!mazeCnt!th"
) ELSE IF !mazeCnt:~-1! EQU 1  ( SET "mm=!mazeCnt!st"
) ELSE IF !mazeCnt:~-1! EQU 2  ( SET "mm=!mazeCnt!nd"
) ELSE IF !mazeCnt:~-1! EQU 3  ( SET "mm=!mazeCnt!rd"
) ELSE SET "mm=!mazeCnt!th"
SET "mm= %mm% maze in !TS_!"
SET "mm=!mm:~0,%wide%!
IF DEFINED logFile ( ECHO(!mm:~1!>>"%logFile%"
	ECHO(>>"!logFile!"
)

:: flashy exit
IF !display! NEQ 0 ( !clear!
	COLOR 3F
	ECHO(!mz:=%xWall%!
	%EKO%!mm:~1!
	FOR %%A IN (3 4 5 1 0) DO FOR %%B IN (%%AF %%A7 %%A8 %%A7) DO (
		COLOR %%B
		CALL :wait 25
		IF NOT EXIST "%keyFile%" GOTO :mazing_restart
	)
)

:mazing_restart
SET return=mCh=!mCh!" "mazeCnt=!mazeCnt!" "clear=!clear!
FOR /L %%A IN (0,1,%opCnt%) DO SET return=!return!" "mzOp%%A=!mzOp%%A!" "mView%%A=!mView%%A!
ENDLOCAL & FOR %%A IN ("%return%") DO SET %%A
GOTO :mazing_loop

:mazing_stop
DEL /F /Q /A "%keyFile%" >NUL 2>&1
>"!cfgFile!" ECHO(v%version%
FOR /L %%A IN (0,1,%opCnt%) DO >>"!cfgFile!" ECHO(mzOp%%A=!mzOp%%A!
CALL :timeSince %totalTime%
IF DEFINED logFile ECHO(_______________________________________________________________________________>>"%logFile%"
IF DEFINED logFile ECHO(%mazeCnt% mazes in !TS_! >>"%logFile%"
TITLE %mazeCnt% mazes in !TS_!
ENDLOCAL
EXIT /B 0

:mazing_hunt_kill stackType rsBias hvBias
:: %1=stack direction. 0=FIFO, else=LIFO
:: %2=next node bias. 0=always random, 100=always stack, >100=oldest/newest
:: %3=directional bias. 1=most vertical, 99=most horizontal
:: If %1<>0 and %2=100 then algorithm is Depth-First Backtracker.
:: If %2>100 then stack will alternate between oldest and newest entries.

SET /A "stk=%~1, curPos=bgnPos, nodes=cols*rows, lbClr=cnt=c1=0, nCnt=trail=far=1, d1=100-%~2, vBias=100-%~3, hBias=%~3, t1=curPos+1, cTmp=wide-4, exClr=!RANDOM!%%(colorCnt-2)"
IF %~3 GTR 99 SET /A "vBias=1, hBias=99"
IF %~3 LSS 1 SET /A "vBias=99, hBias=1"
SET "title0=Hunt & Kill"
IF %~1 NEQ 0 IF %~2 EQU 100 SET title0=Backtracker

:: set bottom message
IF %~1 EQU 0 ( SET "msg= FIFO"
) ELSE SET "msg= LIFO"
IF %~2 LSS 1 ( SET "msg=!msg!:hunt=always:stack=never"
) ELSE IF %~2 EQU 100 ( SET "msg=!msg!:hunt=never:stack=always"
) ELSE IF %~2 GTR 100 ( SET "msg=!msg!:hunt=never:stack=old/new"
) ELSE SET "msg=!msg!:hunt=!d1!%%:stack=%~2%%"
SET "msg=!msg!:vrt=!vBias!%%:hrz=!hBias!%%"
SET "labelTop= %title0% %cols%x%rows% "
SET "mm=!msg:~1!"
SET "msg=!msg:~0,%wide%!"

:: fill maze with 8186 walls, clip to size, and open starting position
SET "labelTop=!labelTop:~0,%cTmp%!"
SET "mz=!wall!"
FOR /L %%A IN (1,1,6) DO SET "mz=!mz!!mz!!mz!!mz!"
SET "top=!mz:~-%wide%!"
SET "mz=!mz:~3!!mz:~3!"
SET "mz=!mz:~0,%size%!"
SET "mz=!mz:~0,%curPos%!!player!!mz:~%t1%!"
SET "r1=!wall!!wall!!labelTop!!top!"
SET "mazeTop=!r1:~0,%wide%!"
SET "mz=!mazeTop!!mz:~%wide%!"

:: set display
IF !display! NEQ 0 (!clear!
	ECHO(!mz!
	%EKO%!msg:~1!
)
IF !display! GTR 0 CALL :mazing_BGcolor

:: build directional bias lists
FOR %%A IN (n s e w) DO SET "%%ABias=%%A"%=                                  'start each list with a single character =%
FOR /L %%A IN (2,1,!hBias!) DO SET "eBias=!eBias!e" & SET "wBias=!wBias!w"%= 'stack characters to represent percentage =%
FOR /L %%A IN (2,1,!vBias!) DO SET "nBias=!nBias!n" & SET "sBias=!sBias!s"%= 'chance to select each direction =%
SET "cp=...!curPos!"%=                                                       'set cp=currentPosition to beginPosition =%
SET "stack= !cp:~-4! "%=                                                     'clip to 4-digits w/leading space+dots+trailing space, stack positions are always 5 characters long =%
FOR /L %%? IN (1,1,64) DO IF !nCnt! LSS !nodes! FOR /L %%@ IN (1,1,64) DO IF !nCnt! LSS !nodes! (
	IF NOT EXIST "%keyFile%" CALL :mazing_menu
	IF ERRORLEVEL 2 EXIT /B !ERRORLEVEL!
	IF !delay! GTR 0 CALL :wait !delay!
	SET "rand="
	IF NOT DEFINED curPos (%= 'trail=#of positions in stack, ch=randomPosition in stack, cTmp=0-99 stackType selection, t1=odd/even stackCount =%
		SET /A "rCnt=0, ch=!RANDOM!%%trail*5, cTmp=!RANDOM!%%100, t1=cnt%%2"
		IF !cTmp! LSS %~2 ( SET "tList=!stack:~0,5!"%=             'tList=topOfStack=lastStackPosition=LIFO =%
			IF %~2 GTR 100 IF !t1! EQU 0 SET "tList=!stack:~-5!"%= 'tList=bottomOfStack=firstStackPosition=FIFO =%
			SET /A "curPos=!tList:.=!"%=                           'remove dots and assure numeracy =%
		) ELSE FOR %%A IN (!ch!) DO FOR /F "tokens=1 delims=. " %%B IN ("!stack:~%%A!") DO SET "curPos=%%B"
	)%=       'or choose randomly using ch and separating newPosition by delimiting dots and spaces in the stack =%
	SET /A "rCnt=rTmp=0, cnt+=1, cTmp=curPos+1, np=curPos-wide*2, sp=curPos+wide*2, ep=curPos+2, wp=curPos-2, wChk=curPos/wide*wide, eChk=wChk+wide, nw=curPos-wide, sw=curPos+wide, ew=curPos+1, ww=curPos-1"
	FOR /F "tokens=1-4" %%A IN ("!np! !sp! !ep! !wp!") DO (%= 'cnt=#of entries in stack, np/sp/ep/wp=north/south/east/west hallPosition, nw/sw/ew/ww=north/south/east/west wallPosition, wChk/eChk=east/west check for maze border =%
		IF !np! GTR !wide! IF "!mz:~%%A,1!" EQU "!wall!" SET /A rCnt+=vBias & SET "rand=!nBias!!rand!"
		IF !sp! LSS !size! IF "!mz:~%%B,1!" EQU "!wall!" SET /A rCnt+=vBias & SET "rand=!sBias!!rand!"
		IF !ep! LSS !eChk! IF "!mz:~%%C,1!" EQU "!wall!" SET /A rCnt+=hBias & SET "rand=!eBias!!rand!"
		IF !wp! GTR !wChk! IF "!mz:~%%D,1!" EQU "!wall!" SET /A rCnt+=hBias & SET "rand=!wBias!!rand!"
	)%= 'build random selection list by including biasList for each direction that is within the maze borders and isn't already a hall =%
	IF !rCnt! NEQ 0 (%= 'new adjacent unvisited cell available =%
		SET /A "nCnt+=1, pct=nCnt*100/nodes, trail+=1, rChk=!RANDOM!%%rCnt"%= 'nCnt=nodeCount=#of nodes added to maze, trail=#of entries in stack, rChk=random entry in selection list =%
		FOR %%A IN (!rChk!) DO FOR %%B IN (!rand:~%%A^,1!) DO SET /A "newPos=!%%Bp!, nTmp=newPos+1, mw=!%%Bw!, mTmp=mw+1"
		SET "cp=...!newPos!"%= 'snag random character from selection list and use it to set directional variables =%
		IF %~1 EQU 0 (%=                      'param1=0=stack is FIFO, so place newPosition on bottom of stack =%
			SET "stack=!stack!!cp:~-4! "
		) ELSE SET "stack= !cp:~-4!!stack!"%= 'else it's LIFO, so place on top instead =%
		FOR /F "tokens=1-4" %%A IN ("!mw! !mTmp! !newPos! !nTmp!") DO (
			SET "mz=!mz:~0,%%A!!hall!!mz:~%%B!"%= 'replace walls with hall+crumb characters in the newPositions =%
			SET "mz=!mz:~0,%%C!!crumb!!mz:~%%D!"
			IF !display! GTR 0 ( SET /A "r1=%%A/wide, c1=%%A-r1*wide, r2=%%C/wide, c2=%%C-r2*wide"
				BG.EXE FCPrint !r1! !c1! !bgClr!!exClr! "!hall!"
				BG.EXE FCPrint !r2! !c2! !bgClr!!exClr! "!crumb!"
			)
		)
	)%= 'remove dead cell from the list =%
	IF !rCnt! LSS 2 (%= 'rCnt<2 means either 0 or 1 directions available from currentPos, either way remove node from stack =%
		SET /A trail-=1
		SET "cp=...!curPos!"%= 'reduce stack count, remove currentPosition from stack =%
		FOR /F "tokens=1-3" %%A IN ("!curPos! !cTmp! !cp:~-4!") DO (
			SET "stack=!stack: %%C = !"
			IF !curPos! NEQ !bgnPos! (%= 'replace crumb character with hall, but not if start/player position =%
				SET "mz=!mz:~0,%%A!!hall!!mz:~%%B!"
				IF !display! GTR 0 ( SET /A "r1=%%A/wide, c1=%%A-r1*wide"
					BG.EXE FCPrint !r1! !c1! !bgClr!!exClr! "!hall!"
				)
			)
		)
	)
	IF !display! LSS 0 ( !clear!
		ECHO(!mz!
		%EKO%!msg:~1!
	)%=    'check for user menu, animate by clearing and EKOing maze =%
	IF !trail! GTR !far! SET /A far=trail%= 'set highest stack count =%
	TITLE !title0! !pct!%% nodes:!nodes!/!nCnt! top:!far!/!trail! #!cnt!   !title1!
	SET /A curPos=newPos%=      'rCnt=0 means no new direction found, so clear currentPosition to force new position from stack =%
	IF !rCnt! EQU 0 SET curPos=
)
:: clear crumbs and exit
SET "mz=!mz:%crumb%=%hall%!"
EXIT /B 0

:mazing_growing_tree stackSize rsBias hvBias
:: %1=size of list for selecting cells. 0=all, >0=LIFO, <0=FIFO.
:: %2=next node bias. 0=always random, 100=always stack, >100=oldest/newest
:: %3=directional bias. 1=most vertical, 99=most horizontal.
:: If %1>0 and %2=100 then algorithm is Depth-First Backtracker.
:: If %2>100 then stack will alternate between oldest and newest entries.

SET "stk="
SET /A "lst=%~1, curPos=bgnPos, nodes=cols*rows, lbClr=cnt=c1=0, nCnt=trail=far=1, d1=100-%~2, vBias=100-%~3, hBias=%~3, t1=curPos+1, cTmp=wide-4, exClr=!RANDOM!%%(colorCnt-2)"
IF !lst! LSS 0 SET /A "stk=lst*=-1"
IF %~3 GTR 99 SET /A "vBias=1, hBias=99"
IF %~3 LSS 1 SET /A "vBias=99, hBias=1"
SET title0=Growing Tree
IF %~1 EQU 1 IF %~2 EQU 100 SET title0=Backtracker

:: set bottom message
IF %~1 EQU 0 ( SET "msg= LIFO list=all"
) ELSE IF %~1 GTR 0 ( SET "msg= LIFO list=!lst!"
) ELSE SET "msg= FIFO list=!lst!"
IF %~2 LSS 1 ( SET "msg=!msg!:random=always:stack=never"
) ELSE IF %~2 EQU 100 ( SET "msg=!msg!:random=never:stack=always"
) ELSE IF %~2 GTR 100 ( SET "msg=!msg!:random=never:stack=old/new"
) ELSE SET "msg=!msg!:random=!d1!%%:stack=%~2%%"
SET "msg=!msg!:vrt=!vBias!%%:hrz=!hBias!%%"
SET "labelTop= %title0% %cols%x%rows% "
SET "mm=!msg:~1!"
SET "msg=!msg:~0,%wide%!"

:: fill maze with 8186 walls, clip to size, and open starting position
SET "labelTop=!labelTop:~0,%cTmp%!"
SET "mz=!wall!"
FOR /L %%A IN (1,1,6) DO SET "mz=!mz!!mz!!mz!!mz!"
SET "top=!mz:~-%wide%!"
SET "mz=!mz:~3!!mz:~3!"
SET "mz=!mz:~0,%size%!"
SET "mz=!mz:~0,%curPos%!!player!!mz:~%t1%!"
SET "r1=!wall!!wall!!labelTop!!top!"
SET "mazeTop=!r1:~0,%wide%!"
SET "mz=!mazeTop!!mz:~%wide%!"

:: set display
IF !display! NEQ 0 (!clear!
	ECHO(!mz!
	%EKO%!msg:~1!
)
IF !display! GTR 0 CALL :mazing_BGcolor

:: build directional bias lists
FOR %%A IN (n s e w) DO SET "%%ABias=%%A"%=                                  'start each list with a single character =%
FOR /L %%A IN (2,1,!hBias!) DO SET "eBias=!eBias!e" & SET "wBias=!wBias!w"%= 'stack characters to represent percentage =%
FOR /L %%A IN (2,1,!vBias!) DO SET "nBias=!nBias!n" & SET "sBias=!sBias!s"%= 'chance to select each direction =%
SET "cp=...!curPos!"%=                                                       'set cp=currentPosition to beginPosition =%
SET "stack= !cp:~-4! "%=                                                     'clip to 4-digits w/leading space+dots+trailing space, stack positions are always 5 characters long =%
:: start 4096 loops/until all nodes are visited, just like Hunt & Kill with different list selection
FOR /L %%? IN (1,1,64) DO IF !nCnt! LSS !nodes! FOR /L %%@ IN (1,1,64) DO IF !nCnt! LSS !nodes! (
	IF NOT EXIST "%keyFile%" CALL :mazing_menu
	IF ERRORLEVEL 2 EXIT /B !ERRORLEVEL!
	IF !delay! GTR 0 CALL :wait !delay!
	SET "rand="
	SET /A limit=trail
	IF !lst! NEQ 0 IF !trail! GTR !lst! SET /A limit=lst
	REM choose cell from visited list
	SET /A "rCnt=0, ch=!RANDOM!%%limit*5, cTmp=!RANDOM!%%100, t1=cnt%%2"
	IF !cTmp! LSS %~2 ( SET "tList=!stack:~0,5!"
		IF %~2 GTR 100 IF !t1! EQU 0 SET "tList=!stack:~-5!"
		SET /A "curPos=!tList:.=!"
	) ELSE FOR %%A IN (!ch!) DO FOR /F "tokens=1 delims=. " %%B IN ("!stack:~%%A!") DO SET "curPos=%%B"
	SET /A "rCnt=rTmp=0, cnt+=1, cTmp=curPos+1, np=curPos-wide*2, sp=curPos+wide*2, ep=curPos+2, wp=curPos-2, wChk=curPos/wide*wide, eChk=wChk+wide, nw=curPos-wide, sw=curPos+wide, ew=curPos+1, ww=curPos-1"
	REM examine NSEW, set highest stack position
	FOR /F "tokens=1-4" %%A IN ("!np! !sp! !ep! !wp!") DO (
		IF !np! GTR !wide! IF "!mz:~%%A,1!" EQU "!wall!" SET /A rCnt+=vBias & SET "rand=!nBias!!rand!"
		IF !sp! LSS !size! IF "!mz:~%%B,1!" EQU "!wall!" SET /A rCnt+=vBias & SET "rand=!sBias!!rand!"
		IF !ep! LSS !eChk! IF "!mz:~%%C,1!" EQU "!wall!" SET /A rCnt+=hBias & SET "rand=!eBias!!rand!"
		IF !wp! GTR !wChk! IF "!mz:~%%D,1!" EQU "!wall!" SET /A rCnt+=hBias & SET "rand=!wBias!!rand!"
	)
	IF !rCnt! NEQ 0 ( REM adjacent unvisited cell available
		REM choose RANDOM direction from list, set new cursor position, add to stack, and write to maze
		SET /A "nCnt+=1, pct=nCnt*100/nodes, trail+=1, rChk=!RANDOM!%%rCnt"
		FOR %%A IN (!rChk!) DO FOR %%B IN (!rand:~%%A^,1!) DO SET /A "newPos=!%%Bp!, nTmp=newPos+1, mw=!%%Bw!, mTmp=mw+1"
		SET "cp=...!newPos!"
		IF DEFINED stk (
			SET "stack=!stack!!cp:~-4! "
		) ELSE SET "stack= !cp:~-4!!stack!"
		FOR /F "tokens=1-4" %%A IN ("!mw! !mTmp! !newPos! !nTmp!") DO (
			SET "mz=!mz:~0,%%A!!hall!!mz:~%%B!"
			SET "mz=!mz:~0,%%C!!crumb!!mz:~%%D!"
			IF !display! GTR 0 ( SET /A "r1=%%A/wide, c1=%%A-r1*wide, r2=%%C/wide, c2=%%C-r2*wide"
				BG.EXE FCPrint !r1! !c1! !bgClr!!exClr! "!hall!"
				BG.EXE FCPrint !r2! !c2! !bgClr!!exClr! "!crumb!"
			)
		)
	)
	IF !rCnt! LSS 2 ( REM remove dead cell from the list
		REM clear crumbs from maze, remove curPos from stack, and set new position
		SET /A trail-=1
		SET "cp=...!curPos!"
		FOR /F "tokens=1-3" %%A IN ("!curPos! !cTmp! !cp:~-4!") DO (
			SET "stack=!stack: %%C = !"
			IF !curPos! NEQ !bgnPos! (
				SET "mz=!mz:~0,%%A!!hall!!mz:~%%B!"
				IF !display! GTR 0 ( SET /A "r1=%%A/wide, c1=%%A-r1*wide"
					BG.EXE FCPrint !r1! !c1! !bgClr!!exClr! "!hall!"
				)
			)
		)
	)
	IF !display! LSS 0 ( !clear!
		ECHO(!mz!
		%EKO%!msg:~1!
	)
	IF !trail! GTR !far! SET "far=!trail!"
	TITLE !title0! !pct!%% nodes:!nodes!/!nCnt! top:!far!/!trail! #!cnt!   !title1!
)
:: clear crumbs and exit
SET "mz=!mz:%crumb%=%hall%!"
EXIT /B 0

:mazing_itr_division wallOpen stackType hvBias
:: %1=open walls where? 0=NW, 1=SE, 2=alternate NW/SE, 
::                      3=midNW, 4=midSE, 5+=random
:: %2=stack order. 0-7=static pre-sets, 8+=random.
:: %3=directional bias. 1=most vertical, 99=most horizontal.
:: if %3=50 walls are generated evenly (ie. vrt if W>H, hrz if H>W).

SET /A "mode=%~3%%20, nodes=cols*rows, lbClr=cnt=nCnt=pct=hPos=0, cTmp=wide-4"
SET /A "b1=bgnPos+1, p1=%~1, t1=rows-1, t2=cols-1, vBias=100-%~3, hBias=%~3"
SET "fill=!hall!"
IF !mode! EQU 0 SET "fill=!crumb!"
IF "!wall!" EQU "" SET "fill=!crumb!"
IF %~3 GTR 99 SET /A "vBias=1, hBias=99"
IF %~3 LSS 1 SET /A "vBias=99, hBias=1"
IF !bgClr! LSS 7 SET "lbClr=F"
SET "title0=Itr. Division"

:: set bottom message
IF %~1 EQU 0 ( SET "msg= wallOpen=NW"
) ELSE IF %~1 EQU 1 ( SET "msg= wallOpen=SE"
) ELSE IF %~1 EQU 2 ( SET "msg= wallOpen=NW/SE"
) ELSE IF %~1 EQU 3 ( SET "msg= wallOpen=mid"
) ELSE SET "msg= wallOpen=random"
IF %~2 LSS 8 ( SET "msg=!msg!:stack=static%~2"
) ELSE SET "msg=!msg!:stack=random"
IF %~3 EQU 50 ( SET "msg=!msg!:vrt=even:hrz=even"
) ELSE SET "msg=!msg!:vrt=!vBias!%%:hrz=!hBias!%%"
SET "mm=!msg:~1!"
SET "msg=!msg:~0,%wide%!"

:: create empty maze with borders
SET "top=!wall!"
FOR /L %%A IN (1,1,5) DO SET "top=!top!!top!!top!!top!"
SET "top=!top:~-%wide%!"
SET "mazeTop= Itr. Division %cols%x%rows% "
SET "labelTop=!mazeTop:~0,%cTmp%!"
SET "mazeTop=!wall!!wall!!labelTop!!top!"
SET "mazeTop=!mazeTop:~0,%wide%!"
SET "mid=!top:%wall%=%fill%!"
SET "mid=!wall!!mid:~2!!wall!"
SET "mz=!mazeTop!"
FOR /L %%A IN (3,1,%high%) DO SET "mz=!mz!!mid!"
SET "mz=!mz:~0,%bgnPos%!!player!!mz:~%b1%!!top!"

:: set display
IF !display! NEQ 0 (!clear!
	ECHO(!mz!
	%EKO%!msg:~1!
)
IF !display! GTR 0 CALL :mazing_BGcolor

:: 0=NS/WE-LIFO, +1=SN/EW, +2=NS/SN-FIFO, +4=WE/EW-FIFO
SET /A "r0=%~2 & 1, r1=(%~2 & 2)/2, r2=(%~2 & 4)/4, block=(nodes+3)/4"
SET "stack=0:0:!t1!:!t2! "

:: ULrow ULcolumn #wallsDown #wallsRight
FOR /L %%? IN (1,1,64) DO IF DEFINED stack FOR /L %%@ IN (1,1,64) DO IF DEFINED stack (
	SET /A cnt+=1
	IF %~1 EQU 2 SET /A "p1=(p1+1)&1"
	FOR /F "tokens=1-4 delims=: " %%A IN ("!stack!") DO (
		IF NOT EXIST "%keyFile%" CALL :mazing_menu
		IF ERRORLEVEL 2 EXIT /B !ERRORLEVEL!
		IF !delay! GTR 0 CALL :wait !delay!
		SET "stack=!stack:* =!"
		SET /A "pct=nCnt*100/block, hv=!RANDOM!%%100"
		IF %~2 GEQ 8 SET /A "r0=!RANDOM! & 1, r1=!RANDOM! & 1, r2=!RANDOM! & 1"
		IF %~3 EQU 50 IF %%C GTR %%D ( SET "hv=0" ) ELSE IF %%C LSS %%D ( SET "hv=99" )
		IF !hv! LSS %~3 ( REM horizontal wall
			IF !p1! EQU 0 ( SET hPos=0
			) ELSE IF !p1! EQU 1 ( SET hPos=%%D
			) ELSE IF !p1! EQU 3 ( SET /A "hPos=(%%D+!RANDOM!%%2)/2"
			) ELSE SET /A "hPos=!RANDOM!%%(%%D+1)"
			SET /A "wPos=!RANDOM!%%%%C+%%A+1, wLen=%%D*2, rLen=wLen-hPos*2, lLen=wLen-rLen"
			SET /A "sPos=wPos*2*wide+%%B*2+1, ePos=sPos+wLen+1, t1=wPos-%%A-1, t2=%%C-t1-1"
			FOR /F "tokens=1-4" %%E IN ("!lLen! !rLen! !sPos! !ePos!") DO (
				SET "mz=!mz:~0,%%G!!top:~0,%%E!!fill!!top:~0,%%F!!mz:~%%H!"
			)
			IF !r0! EQU 0 ( REM randomizing the stack, could replace with a single line
				IF !t1! NEQ 0 ( SET /A "nCnt+=1"
					IF !r1! EQU 0 ( SET "stack=%%A:%%B:!t1!:%%D !stack!"
					) ELSE SET "stack=!stack!%%A:%%B:!t1!:%%D ")
				IF !t2! NEQ 0 ( SET /A "nCnt+=1"
					IF !r2! EQU 0 ( SET "stack=!wPos!:%%B:!t2!:%%D !stack!"
					) ELSE SET "stack=!stack!!wPos!:%%B:!t2!:%%D ")
			) ELSE (
				IF !t2! NEQ 0 ( SET /A "nCnt+=1"
					IF !r1! EQU 0 ( SET "stack=!wPos!:%%B:!t2!:%%D !stack!"
					) ELSE SET "stack=!stack!!wPos!:%%B:!t2!:%%D ")
				IF !t1! NEQ 0 ( SET /A "nCnt+=1"
					IF !r2! EQU 0 ( SET "stack=%%A:%%B:!t1!:%%D !stack!"
					) ELSE SET "stack=!stack!%%A:%%B:!t1!:%%D ")
			)
		) ELSE ( REM vertical wall
			IF !p1! EQU 0 ( SET hPos=0
			) ELSE IF !p1! EQU 1 ( SET hPos=%%C
			) ELSE IF !p1! EQU 3 ( SET /A "hPos=(%%C+!RANDOM!%%2)/2"
			) ELSE SET /A "hPos=!RANDOM!%%(%%C+1)"
			SET /A "wPos=!RANDOM!%%%%D+%%B+1, wLen=%%C*2, rLen=hPos*2, t1=wPos-%%B-1, t2=%%D-t1-1"
			SET /A "sPos=(%%A*2+1)*wide+wPos*2, ePos=sPos+wLen*wide, hCnt=-1"
			FOR /L %%E IN (!sPos!,!wide!,!ePos!) DO (
				SET /A "mTmp=%%E+1, hCnt+=1"
				IF !hCnt! NEQ !rLen! FOR %%F IN (!mTmp!) DO SET "mz=!mz:~0,%%E!!wall!!mz:~%%F!"
			)
			IF !r0! EQU 0 (
				IF !t1! NEQ 0 ( SET /A "nCnt+=1"
					IF !r1! EQU 0 ( SET "stack=%%A:%%B:%%C:!t1! !stack!"
					) ELSE SET "stack=!stack!%%A:%%B:%%C:!t1! ")
				IF !t2! NEQ 0 ( SET /A "nCnt+=1"
					IF !r2! EQU 0 ( SET "stack=%%A:!wPos!:%%C:!t2! !stack!"
					) ELSE SET "stack=!stack!%%A:!wPos!:%%C:!t2! ")
			) ELSE (
				IF !t2! NEQ 0 ( SET /A "nCnt+=1"
					IF !r1! EQU 0 ( SET "stack=%%A:!wPos!:%%C:!t2! !stack!"
					) ELSE SET "stack=!stack!%%A:!wPos!:%%C:!t2! ")
				IF !t1! NEQ 0 ( SET /A "nCnt+=1"
					IF !r2! EQU 0 ( SET "stack=%%A:%%B:%%C:!t1! !stack!"
					) ELSE SET "stack=!stack!%%A:%%B:%%C:!t1! ")
			)
		)
	)
	IF !display! GTR 0 ( BG.EXE Locate 1 0
		ECHO(!mz:~%wide%!
	)
	IF !display! LSS 0 ( !clear!
		ECHO(!mz!
		%EKO%!msg:~1!
	)
	IF !nCnt! LEQ !block! TITLE !title0! !pct!%% block:!block!/!nCnt! #!cnt!   !title1!
)
TITLE !title0! 100%% block:!block!/!block! #!cnt!   !title1!
SET "mz=!mz:%crumb%=%hall%!"
EXIT /B 0

:mazing_ellers hvBias
:: %1=directional bias. 0=most vertical, 99=most horizontal
SET /A "mode=%~1%%20, curPos=bgnPos, b1=bgnPos+1, b2=bgnPos/(wide*2)+1, lbClr=sCnt=cnt=c1=0, nCnt=trail=far=1, cTmp=wide-4, vBias=100-%~1, hBias=%~1"
SET "fill=!hall!"
IF !mode! EQU 0 SET "fill=!crumb!"
IF "!wall!" EQU "" SET "fill=!crumb!"
IF %~1 GTR 99 SET /A "vBias=1, hBias=99"
IF %~1 LSS 1 SET /A "vBias=99, hBias=1"
SET "msg= vrt=!vBias!%%:hrz=!hBias!%%"
SET "mm=!msg:~1!"
SET "title0=Eller's Algor"

:: create border and mazeTop
SET "top=!wall!"
FOR /L %%A IN (1,1,5) DO SET "top=!top!!top!!top!!top!"
SET "top=!top:~-%wide%!"
SET "mazeTop= Eller's Algor %cols%x%rows% "
SET "labelTop=!mazeTop:~0,%cTmp%!"
SET "mazeTop=!wall!!wall!!labelTop!!top!"
SET "mazeTop=!mazeTop:~0,%wide%!"
SET "wlz=!mazeTop!"
SET "hlz="
SET "mz="

IF !display! NEQ 0 (!clear!
	COLOR !bgClr!!fgClr!
	ECHO(!wlz!
)
IF !display! GTR 0 CALL :mazing_BGcolor

FOR /L %%A IN (1,1,%cols%) DO SET "c%%A="
FOR /L %%A IN (1,1,%rows%) DO (
	IF NOT EXIST "%keyFile%" CALL :mazing_menu
	IF ERRORLEVEL 2 EXIT /B !ERRORLEVEL!
	IF !delay! GTR 0 CALL :wait !delay!
	SET "mz=!mz!!wlz!"
	IF !display! GTR 0 ( BG.EXE Locate 1 0
		ECHO(!mz:~%wide%!
	)
	IF !display! LSS 0 ( !clear!
		ECHO(!mz!
	)
	FOR /L %%B IN (1,1,%cols%) DO IF NOT DEFINED c%%B SET /A "c%%B=sCnt, sCnt+=1"
	SET "hlz=!wall!!fill!"
	SET "stack=;!c1! "
	SET /A "last=c1, pct=%%A*100/rows"
	TITLE !title0! !pct!%% hrz=!hBias!%%/vrt=!vBias!%%   !title1!
	FOR /L %%B IN (2,1,%cols%) DO (
		SET /A "rc=!RANDOM!%%100"
		IF %%A EQU %rows% SET rc=0
		IF !rc! LSS !hBias! (
			IF !c%%B! NEQ !last! (
				SET /A rc=c%%B
				FOR /L %%C IN (1,1,%cols%) DO IF !c%%C! EQU !rc! SET /A "c%%C=last"
				SET "hlz=!hlz!!fill!!fill!"
			) ELSE SET "hlz=!hlz!!wall!!fill!"
		) ELSE SET "hlz=!hlz!!wall!!fill!"
		SET "stack=!stack!;!c%%B! "
		SET /A "last=c%%B"
	)
	SET "mz=!mz!!hlz!!wall!"
	IF %%A EQU !b2! SET "mz=!mz:~0,%bgnPos%!!player!!mz:~%b1%!"
	IF !display! GTR 0 ( BG.EXE Locate 1 0
		ECHO(!mz:~%wide%!
	)
	IF !display! LSS 0 ( !clear!
		ECHO(!mz!
	)
	SET "stack=!stack!A"
	SET "wlz=!wall!"
	IF %%A NEQ %rows% FOR /L %%B IN (1,1,%cols%) DO (
		SET /A "rr=!RANDOM!%%100"
		SET "stack=!stack:* =!"
		FOR %%C IN (!c%%B!) DO IF "!stack:;%%C =!" EQU "!stack!" SET "rr=99"
		IF !rr! LSS !hBias! (
			SET "c%%B="
			SET "wlz=!wlz!!wall!!wall!"
		) ELSE (
			SET "stack=!stack!;!c%%B! "
			SET "wlz=!wlz!!fill!!wall!"
		)
	)
)
SET "mz=!mz!!top!"
SET "mz=!mz:%crumb%=%hall%!"
IF !display! GTR 0 ( BG.EXE Locate 1 0
	ECHO(!mz:~%wide%!
	BG.EXE Print !bgClr!!lbClr! "!msg:~1!"
)
EXIT /B 0

:mazing_rebuild boxType
:: %1=type of maze produced. %1<0=random single character, %1>0=box characters 1-12
SET /A "wd=cols*2, bx=%~1%%13, t1=!RANDOM!%%bColorCnt, t2=!RANDOM!%%fColorCnt, t3=!RANDOM!%%wallCnt, b1=bgnPos+1, e1=endPos+1"
IF !mzOp14! LSS 0  SET "bgClr=!bColors:~%t1%,1!"
IF !mzOp14! GTR 15 SET "bgClr=!bColors:~%t1%,1!"
IF !mzOp15! LSS 0  SET "fgClr=!fColors:~%t2%,1!"
IF !mzOp15! GTR 15 SET "fgClr=!fColors:~%t2%,1!"
IF "!bgClr!" EQU "!fgClr!" SET "fgClr=D"
FOR %%A IN (!badClrs!) DO IF "!bgClr!!fgClr!" EQU "%%A" (
	SET "bgClr=!goodClr:~0,1!" & SET "fgClr=!goodClr:~-1!")
SET "mTmp=!walls:~%t3%,1!"
SET "title0=Convert Box#%~1"

:: change color and wall character and exit
IF %~1 LSS 0 (
	SET "mz=!mz:%wall%=%mTmp%!"
	SET "wall=!mTmp!"
	SET "top=!mz:~-%wide%!"
	SET "mazeTop=!wall!!wall!!labelTop!!top!"
	SET "mazeTop=!mazeTop:~0,%wide%!"
	SET "mz=!mazeTop!!mz:~%wide%!
	IF !display! NEQ 0 ( !clear!
		COLOR !bgClr!!fgClr!
		ECHO(!mz!
		%EKO%!msg:~1!
	)
	EXIT /B 0
)

:: clear special characters from maze
SET "mz=!mz:%player%=%hall%!"
SET "mz=!mz:%goal%=%hall%!"
SET "mz=!top!!mz:~%wide%!"
IF !display! NEQ 0 ( !clear!
	COLOR !fgClr!!bgClr!
	ECHO(!mz!
	%EKO%!msg:~1!
)
:: check every character and replace walls with box characters
FOR /L %%? IN (0,%wide%,%size%) DO IF %%? LSS %size% (
	SET /A "et=%%?+wide, pct=%%?*100/size"
	IF NOT EXIST "%keyFile%" CALL :mazing_menu
	IF ERRORLEVEL 2 EXIT /B !ERRORLEVEL!
	TITLE !title0! !pct!%%   !title1!
	FOR /L %%@ IN (0,1,%wd%) DO (
		SET /A "cp=%%?+%%@, ct=cp+1, nw=cp-wide, sw=cp+wide, ew=cp+1, ww=cp-1, list=-1"
		FOR /F "tokens=1-6" %%A IN ("!nw! !sw! !ew! !ww! !cp! !ct!") DO IF "!mz:~%%E,1!" EQU "!wall!" (
			IF !nw! GEQ 0      IF "!mz:~%%A,1!" NEQ "!hall!" SET /A list+=1
			IF !sw! LSS !size! IF "!mz:~%%B,1!" NEQ "!hall!" SET /A list+=2
			IF !ew! LSS !et!   IF "!mz:~%%C,1!" NEQ "!hall!" SET /A list+=4
			IF !ww! GEQ %%?    IF "!mz:~%%D,1!" NEQ "!hall!" SET /A list+=8
			IF !list! GEQ 0 FOR %%G IN (!list!) DO SET "mz=!mz:~0,%%E!!box%bx%:~%%G,1!!mz:~%%F!"
		)
	)
	IF %%? GTR !b1! SET b1=!size!& SET "mz=!mz:~0,%bgnPos%!!player!!mz:~%b1%!"
	IF %%? GTR !e1! SET e1=!size!& SET "mz=!mz:~0,%endPos%!!goal!!mz:~%e1%!"
	IF !display! GTR 0 (
		IF %%? EQU 0 ( BG.EXE Locate 0 0
			ECHO(!mz:~0,%wide%!
			CALL :mazing_BGcolor)
		BG.EXE Locate 1 0
		ECHO(!mz:~%wide%!
	)
	IF !display! LSS 0 ( !clear!
		ECHO(!mz!
		%EKO%!msg:~1!
	)
)
TITLE !title0! 100%%   !title1!
SET /A "t1=!RANDOM!%%bColorCnt, t2=!RANDOM!%%fColorCnt, t3=!RANDOM!%%crumbCnt"
SET "mazeTop=!mz:~0,%wide%!  !labelTop!"
SET "mazeTop=!mazeTop:~-%wide%!"
SET "mazeTop=!mz:~0,2!!labelTop!!mazeTop!"
SET "mazeTop=!mazeTop:~0,%wide%!"
SET "mz=!mazeTop!!mz:~%wide%!
::SET "mz=!mz:~0,%bgnPos%!!player!!mz:~%ct%!"
::SET "mz=!mz:~0,%endPos%!!goal!!mz:~%et%!"
IF !mzOp11! LSS 0  SET "crumb=!crumbs:~%t3%,1!"
IF !mzOp14! LSS 0  SET "bgClr=!bColors:~%t1%,1!"
IF !mzOp14! GTR 15 SET "bgClr=!bColors:~%t1%,1!"
IF !mzOp15! LSS 0  SET "fgClr=!fColors:~%t2%,1!"
IF !mzOp15! GTR 15 SET "fgClr=!fColors:~%t2%,1!"
IF "!bgClr!" EQU "!fgClr!" SET "fgClr=D"
FOR %%A IN (!badClrs!) DO IF "!bgClr!!fgClr!" EQU "%%A" (
	SET "bgClr=!goodClr:~0,1!" & SET "fgClr=!goodClr:~-1!")
IF !display! NEQ 0 ( !clear!
	COLOR !bgClr!!fgClr!
	ECHO(!mz!
	%EKO%!msg:~1!
)
EXIT /B 0

:mazing_wall_follow fillType
:: %1=write what to maze? %1<0=walls, %1>=0=halls
SET /A "curPos=bgnPos, nodes=cols*rows, lbClr=c1=0, nCnt=nTmp=cnt=trail=1, cTmp=wide-4, wTmp=!RANDOM!%%2, mTmp=!RANDOM!%%4+1"
IF !wTmp! EQU 0 ( REM pick random starting hand
	SET "hand=Left"
	SET "listN=s e n w"
	SET "listW=e n w s"
	SET "listS=n w s e"
	SET "listE=w s e n"
) ELSE (
	SET "hand=Right"
	SET "listN=s w n e"
	SET "listW=e s w n"
	SET "listS=n e s w"
	SET "listE=w n e s"
)
FOR %%A IN (!listN!) DO (
	IF !mTmp! EQU !nTmp! SET "list=!list%%A!"
	SET /A nTmp+=1
)

SET "title0=Wall Follow"
SET "r1= Wall Follow !hand! "
SET "labelBtm=!r1:~0,%cTmp%!"
SET "r1=!mz:~-%wide%!  !labelBtm!"
SET "r1=!r1:~0,2!!labelBtm!!r1:~-%wide%!"
SET "mazeBtm=!r1:~0,%wide%!"
SET "mz=!mz:~0,-%wide%!!mazeBtm!"
SET "bz=!mz!"
SET "fill=!hall!"
IF %~1 LSS 0 SET "fill=!wall!"

:: set display
IF !display! NEQ 0 ( !clear!
	ECHO(!mz!
	%EKO%!msg:~1!
)
IF !display! GTR 0 CALL :mazing_BGcolor

SET "stack=" & SET "stk="
FOR /L %%? IN (1,1,64) DO IF !curPos! NEQ !endPos! FOR /L %%@ IN (1,1,64) DO IF !curPos! NEQ !endPos! (
	IF NOT EXIST "%keyFile%" CALL :mazing_menu
	IF ERRORLEVEL 2 EXIT /B !ERRORLEVEL!
	IF !delay! GTR 0 CALL :wait !delay!
	SET /A "cnt+=1, cTmp=curPos+1, np=curPos-wide*2, sp=curPos+wide*2, ep=curPos+2, wp=curPos-2, wChk=curPos/wide*wide, eChk=wChk+wide, nw=curPos-wide, sw=curPos+wide, ew=curPos+1, ww=curPos-1"
	FOR %%A IN (!list!) DO FOR /F "tokens=1,2" %%B IN ("!%%Aw! !%%Ap!") DO (
		SET "wTmp=!mz:~%%B,1!"
		IF "!wTmp!" EQU "!crumb!" SET "wTmp=!hall!"
		IF "!wTmp!" EQU "!hall!" IF %%C GTR !wide! IF %%C LSS !size! (
			SET "cw=%%B"
			SET "newPos=%%C"
			SET "newList=!list%%A!"
			SET "char=!mz:~%%B,1!"
		)
	)
	SET /A wTmp=cw+1, nTmp=newPos+1
	IF "!char!" EQU "!crumb!" ( SET /A trail-=1
		SET "stack=!stack:* =!"
		FOR /F "tokens=1-4" %%A IN ("!curPos! !cTmp! !cw! !wTmp!") DO (
			SET "mz=!mz:~0,%%A!!fill!!mz:~%%B!"
			SET "mz=!mz:~0,%%C!!fill!!mz:~%%D!"
			IF !display! GTR 0 ( SET /A "r1=%%A/wide, c1=%%A-r1*wide, r2=%%C/wide, c2=%%C-r2*wide"
				BG.EXE FCPrint !r1! !c1! !bgClr!!ex2Clr! "!fill!"
				BG.EXE FCPrint !r2! !c2! !bgClr!!ex2Clr! "!fill!"
			)	
		)
	) ELSE (
		SET "stack=!newPos! !stack!"
		SET /A "nCnt+=1, pct=nCnt*100/nodes, trail+=1"
		FOR /F "tokens=1-4" %%A IN ("!cw! !wTmp! !newPos! !nTmp!") DO (
			SET "mz=!mz:~0,%%A!!crumb!!mz:~%%B!"
			SET "mz=!mz:~0,%%C!!crumb!!mz:~%%D!"
			IF !display! GTR 0 ( SET /A "r1=%%A/wide, c1=%%A-r1*wide, r2=%%C/wide, c2=%%C-r2*wide"
				BG.EXE FCPrint !r1! !c1! !bgClr!!exClr! "!crumb!"
				BG.EXE FCPrint !r2! !c2! !bgClr!!exClr! "!crumb!"
			)
		)
	)
	IF !trail! GTR !far! SET /A far=trail
	SET /A curPos=newPos
	SET "list=!newList!"
	TITLE !title0! !pct!%% nodes:!nodes!/!nCnt! trail:!trail! #!cnt!   !title1!
	IF !display! LSS 0 ( !clear!
		ECHO(!mz!
		%EKO%!msg:~1!
	)
)
IF !curPos! NEQ !endPos! CALL :doErr
SET /A "b1=bgnPos+1, e1=endPos+1"
IF %~1 LSS 0 (
	SET "mz=!bz!"
	FOR %%A IN (!bgnPos! !stack! !endPos!) DO ( SET /A "t1=%%A+1"
		FOR %%B IN (!t1!) DO SET "mz=!mz:~0,%%A!!crumb!!mz:~%%B!
	)
	FOR %%A IN (!bgnPos! !stack! !endPos!) DO (
		SET /A "sp=%%A+wide*2, ep=%%A+2, sw=%%A+wide, ew=%%A+1"
		FOR /F "tokens=1-4" %%B IN ("!sp! !sw! !ep! !ew!") DO (
			IF "!mz:~%%B,1!" EQU "!crumb!" IF "!mz:~%%C,1!" NEQ "!wall!" SET "stk=!stk!%%C "
			IF "!mz:~%%D,1!" EQU "!crumb!" IF "!mz:~%%E,1!" NEQ "!wall!" SET "stk=!stk!%%E "
		)
	)
	FOR %%A IN (!stk!) DO ( SET /A "t1=%%A+1"
		FOR %%B IN (!t1!) DO (
			SET "mz=!mz:~0,%%A!!crumb!!mz:~%%B!"
		)
	)
)
SET "mz=!mz:~0,%bgnPos%!!player!!mz:~%b1%!"
SET "mz=!mz:~0,%endPos%!!goal!!mz:~%e1%!"
EXIT /B 0

:mazing_dead_filler fillType
:: %1=write what to maze? %1<0=walls, %1>=0=halls
SET /A "dfMode=%~1, nodes=cols*rows, lbClr=nCnt=c1=t1=0, nTmp=cnt=trail=1, cTmp=wide-4, nChk=wide*2, sChk=size-wide*2"
SET "r1= Dead Filler "
SET "labelBtm=!r1:~0,%cTmp%!"
SET "r1=!mz:~-%wide%!  !labelBtm!"
SET "r1=!r1:~0,2!!labelBtm!!r1:~-%wide%!"
SET "mazeTop=!mz:~0,%wide%!"
SET "mazeBtm=!r1:~0,%wide%!"
SET "fix=!crumb!"
SET "fill=!hall!"
IF %~1 LSS 0 (
	SET "fix=!hall!"
	SET "fill=!wall!"
) ELSE SET "mz=!mz:%hall%=%crumb%!"
SET "mz=!mazeTop!!mz:~%wide%,-%wide%!!mazeBtm!"
SET "bz=!mz!"
SET "rw0= " & SET "rw1= "
SET "title0=Dead Filler"

:: set display
IF !display! NEQ 0 ( !clear!
	ECHO(!mz!
	%EKO%!msg:~1!
)
IF !display! GTR 0 CALL :mazing_BGcolor

:: create stacks of cell positions for each row, omit start and end
SET /A cp=np=t1=0, b1=bgnPos+1, e1=endPos+1, stl=cols*6
FOR /L %%A IN (0,1,16) DO ( SET "np%%A=" ) & ( SET "wp%%A=" )
FOR /L %%A IN (1,1,%rows%) DO FOR /L %%B IN (1,1,%cols%) DO (
	SET /A "np=wide*%%A*2-wide+%%B*2-1, t1+=1"
	IF !np! NEQ !bgnPos! IF !np! NEQ !endPos! (
		IF !t1! LSS 1500 ( SET "rw0=!rw0!!np! "
		) ELSE SET "rw1=!rw1!!np! "
	)
)

:: loop through stack looking for cells with only one 'hall'
:: fill it in and remove it from the stack, repeat until no change
FOR /L %%@ IN (1,1,64) DO IF DEFINED t1 ( SET "t1="
	FOR %%A IN (0 1) DO FOR %%B IN (!rw%%A!) DO (
		IF NOT EXIST "%keyFile%" CALL :mazing_menu
		IF ERRORLEVEL 2 EXIT /B !ERRORLEVEL!
		IF !delay! GTR 0 CALL :wait !delay!
		IF !cp! EQU 0 SET /A "cp=%%B"
		SET /A "wp1=cp-wide, wp2=cp+wide, wp4=cp+1, wp8=cp-1, cnt+=1, pow=1"
		SET /A "np1=cp-wide*2, np2=cp+wide*2, np4=cp+2, np8=cp-2, np=cp, cp=wl=0"
		IF !np! LSS !nChk! SET "wp1=0"
		IF !np! GTR !sChk! SET "wp2=0"
		FOR %%C IN (!wp1! !wp2! !wp4! !wp8!) DO (
			IF "!mz:~%%C,1!" EQU "!fix!" SET /A "wl+=pow"
			SET /A "pow*=2"
		)
		FOR %%C IN (!wl!) DO FOR /F "tokens=1-2" %%D IN ("!wp%%C! !np%%C!") DO (
			SET /A "t1=%%D+1, nCnt+=1, pct=nCnt*100/nodes"
			IF %%E NEQ !bgnPos! IF %%E NEQ !endPos! SET /A "cp=%%E"
			FOR /F "tokens=1-3" %%F IN ("!np! !wp4! !t1!") DO (
				SET "mz=!mz:~0,%%F!!fill!!mz:~%%G!"
				SET "mz=!mz:~0,%%D!!fill!!mz:~%%H!"
				SET "rw0=!rw0: %%F = !"
				SET "rw1=!rw1: %%F = !"
			)
			IF !display! GTR 0 (
				SET /A "r1=np/wide, c1=np-r1*wide, r2=!wp%%C!/wide, c2=!wp%%C!-r2*wide"
				BG.EXE FCPrint !r1! !c1! !dfClr! "!fill!"
				BG.EXE FCPrint !r2! !c2! !dfClr! "!fill!"
			)
			IF !display! LSS 0 ( !clear!
				ECHO(!mz!
				%EKO%!msg:~1!
			)
		)
		TITLE !title0! !pct!%% pass#%%@ nodes:!nodes!/!nCnt!/n!np! #!cnt!   !title1!
	)
)
IF DEFINED t1 CALL :doErr
IF !pct! GEQ 99 CALL :doErr
IF %~1 LSS 0 (
	SET "t0= "
	SET "mz=!bz!"
	FOR %%A IN (!bgnPos! !rw0! !rw1! !endPos!) DO ( SET /A "t1=%%A+1"
		FOR %%B IN (!t1!) DO SET "mz=!mz:~0,%%A!!crumb!!mz:~%%B!
	)
	FOR %%A IN (!bgnPos! !rw0! !rw1! !endPos!) DO (
		SET /A "sp=%%A+wide*2, ep=%%A+2, sw=%%A+wide, ew=%%A+1"
		FOR /F "tokens=1-4" %%B IN ("!sp! !sw! !ep! !ew!") DO (
			IF "!mz:~%%B,1!" EQU "!crumb!" IF "!mz:~%%C,1!" NEQ "!wall!" SET "t0=!t0!%%C "
			IF "!mz:~%%D,1!" EQU "!crumb!" IF "!mz:~%%E,1!" NEQ "!wall!" SET "t0=!t0!%%E "
		)
	)
	FOR %%A IN (!t0!) DO ( SET /A "t1=%%A+1"
		FOR %%B IN (!t1!) DO (
			SET "mz=!mz:~0,%%A!!crumb!!mz:~%%B!"
		)
	)
)
SET "mz=!mz:~0,%bgnPos%!!player!!mz:~%b1%!"
SET "mz=!mz:~0,%endPos%!!goal!!mz:~%e1%!"
EXIT /B 0

:mazing_path_finder fillType
:: %1=fill maze with what? %1<0=walls, %1>=0=crumbs
::    If %1 is odd, worms show the current stack.
SET /A "nodes=cols*rows, lbClr=chk=cnt=0, trail=nCnt=1, cTmp=wide-4, mode=%~1 %% 2"
SET "title0=Path Finder"
SET "r1= Path Finder "
SET "labelBtm=!r1:~0,%cTmp%!"
SET "r1=!mz:~-%wide%!  !labelBtm!"
SET "r1=!r1:~0,2!!labelBtm!!r1:~-%wide%!"
SET "mazeTop=!mz:~0,%wide%!"
SET "mazeBtm=!r1:~0,%wide%!"
SET "mz=!mazeTop!!mz:~%wide%,-%wide%!!mazeBtm!"
SET "bz=!mz!"
SET "fill=!crumb!"
IF %~1 LSS 0 SET "fill=!wall!"

:: set display
IF !display! NEQ 0 ( !clear!
	ECHO(!mz!
	%EKO%!msg:~1!
)
IF !display! GTR 0 CALL :mazing_BGcolor

SET "pf%bgnPos%=-1"
SET "stack=!bgnPos! "
FOR /L %%? IN (1,1,64) DO IF NOT DEFINED pf%endPos% FOR /L %%@ IN (1,1,64) DO IF NOT DEFINED pf%endPos% (
	SET "newStk1="
	SET "newStk2="
	SET /A trail+=1
	FOR %%A IN (!stack!) DO ( SET "stack=!stack:* =!"
		IF NOT EXIST "%keyFile%" CALL :mazing_menu
		IF ERRORLEVEL 2 EXIT /B !ERRORLEVEL!
		IF !delay! GTR 0 CALL :wait !delay!
		SET /A "cnt+=1, np=nt=%%A-wide*2, sp=st=%%A+wide*2, ep=et=%%A+2, wp=wt=%%A-2, nw=nq=%%A-wide, sw=sq=%%A+wide, ew=eq=%%A+1, ww=wq=%%A-1, nt+=1, st+=1, et+=1, wt+=1, nq+=1, sq+=1, eq+=1, wq+=1"
		FOR %%B IN (n s e w) DO FOR /F "tokens=1-4" %%C IN ("!%%Bw! !%%Bq! !%%Bp! !%%Bt!") DO (
			IF "!mz:~%%C,1!" EQU "!hall!" IF NOT DEFINED pf%%E IF %%E GTR !wide! IF %%E LSS !size! (
				SET /A "nCnt+=1, pct=nCnt*100/nodes"
				SET "mz=!mz:~0,%%C!!fill!!mz:~%%D!"
				IF %%E NEQ !endPos! SET "mz=!mz:~0,%%E!!fill!!mz:~%%F!"
				SET "stack=!stack!%%E "
				SET "pf%%E=%%C %%D %%A !ew!"
				SET "newStk1=!newStk1!%%C:%%D %%E:%%F "
				IF !display! GTR 0 (
					SET /A "r1=%%C/wide, c1=%%C-r1*wide, r2=%%E/wide, c2=%%E-r2*wide"
					BG.EXE FCPrint !r1! !c1! !bgClr!!exClr! "!fill!"
					IF %%E NEQ !endPos! BG.EXE FCPrint !r2! !c2! !bgClr!!exClr! "!fill!"
					SET "newStk2=!newStk2!!r1!:!c1! !r2!:!c2! "
				)
			)
		)
		TITLE !title0! !pct!%% nodes:!nodes!/!nCnt! trail:!trail! #!cnt!   !title1!
	)
	IF !display! LSS 0 ( !clear!
		ECHO(!mz!
		%EKO%!msg:~1!
	)
	IF !mode! NEQ 0 (
		FOR %%A IN (!crmStk1!) DO FOR /F "tokens=1-2 delims=:" %%B IN ("%%A") DO (
			SET "mz=!mz:~0,%%B!!hall!!mz:~%%C!"
		)
		SET "crmStk1=!newStk1!"
		IF !display! GTR 0 (
			FOR %%A IN (!crmStk2!) DO FOR /F "tokens=1-2 delims=:" %%B IN ("%%A") DO (
				BG.EXE FCPrint %%B %%C !bgClr!!exClr! "!hall!"
			)
			SET "crmStk2=!newStk2!"
		)
		
	)
)
IF NOT DEFINED pf%endPos% CALL :doErr
SET "mz=!bz!"
SET "pf=!pf%endPos%!"
FOR /L %%@ IN (2,1,!trail!) DO FOR /F "tokens=1-4" %%A IN ("!pf!") DO (
	SET "mz=!mz:~0,%%A!!crumb!!mz:~%%B!"
	IF %%@ LSS !trail! SET "mz=!mz:~0,%%C!!crumb!!mz:~%%D!"
	SET "pf=!pf%%C!"
)
EXIT /B 0

:mazing_bgnPos where
:: %1=place entrance where?
:: 0-=random point, 1=random point on border, 2+=random corner
:: bgnPos=maze entrance position, curPos=current cursor position
SET /A "p1=!RANDOM!%%4"
IF %1 LEQ 0 SET /A "bgnPos=(!RANDOM!%%cols*2+1)+(!RANDOM!%%rows*2+1)*wide"
IF %1 EQU 1 ( SET /A "bgnPos=wide+!RANDOM!%%cols*2+1"
	IF !p1! EQU 1 SET /A "bgnPos=size-wide-!RANDOM!%%cols*2-2"
	IF !p1! EQU 2 SET /A "bgnPos=(!RANDOM!%%rows*2+1)*wide+1"
	IF !p1! EQU 3 SET /A "bgnPos=size-(!RANDOM!%%rows*2+1)*wide-2")
IF %1 GEQ 2 ( SET /A "bgnPos=wide+1"
	IF !p1! EQU 1 SET /A "bgnPos=wide*2-2"
	IF !p1! EQU 2 SET /A "bgnPos=size-wide*2+1"
	IF !p1! EQU 3 SET /A "bgnPos=size-wide-2")
SET /A curPos=bgnPos
EXIT /B 0

:mazing_endPos where
:: %1=place exit where?
:: 0=farthest point, 1=farthest point on border, 2+=farthest corner
SETLOCAL
SET /A "cnt=trail=ep%bgnPos%=0"
SET "stack=!bgnPos! "
IF %1 EQU 0 SET title0=Inside
IF %1 EQU 1 SET title0=Border
SET "title2=!title1!"
IF %1 GTR 1 ( SET /A "endPos%1=wide+1, r1=bgnPos/wide, c1=bgnPos-r1*wide"
	IF !r1! LSS !rows! SET /A endPos%1=size-wide*2+1
	IF !c1! LSS !cols! SET /A endPos%1+=wide-3
) ELSE FOR /L %%? IN (1,1,64) DO IF DEFINED stack FOR /L %%@ IN (1,1,64) DO IF DEFINED stack (
	IF NOT EXIST "%keyFile%" SET "title2=Please Wait..."
	FOR %%A IN (!stack!) DO ( SET "stack=!stack:* =!"   'remove opening spaces
		SET /A "cnt+=1, np=%%A-wide*2, sp=%%A+wide*2, ep=%%A+2, wp=%%A-2, nw=%%A-wide, sw=%%A+wide, ew=%%A+1, ww=%%A-1, wk=%%A/wide*wide, ek=wk+wide-1"
		FOR %%B IN (n s e w) DO FOR /F "tokens=1-2" %%C IN ("!%%Bw! !%%Bp!") DO (
			IF %%D LSS !wide! ( SET "endPos1=%%A"
			) ELSE IF %%D GTR !size! ( SET "endPos1=%%A"
			) ELSE IF %%C EQU !wk! ( SET "endPos1=%%A"
			) ELSE IF %%C EQU !ek! ( SET "endPos1=%%A"
			) ELSE IF "!mz:~%%C,1!" EQU "!hall!" IF NOT DEFINED ep%%D (
				SET "stack=!stack!%%D "
				SET "endPos0=%%D"
				SET "ep%%D=1"
			)
		)
	)
	SET /A trail+=1
	TITLE End Position !title0! / trail:!trail! #!cnt!   !title2!
)
SET "endPos=!endPos%1!"
ENDLOCAL & SET /A "endPos=%endPos%, trail=%trail%, t1=endPos+1"
SET "mz=!mz:~0,%endPos%!!goal!!mz:~%t1%!"
IF NOT EXIST "%keyFile%" CALL :mazing_menu
EXIT /B !ERRORLEVEL!

:timeSince %TIME% [%DATE% [part whole]]
:: input  - hh:mm:ss.cc [Day MM/DD/YYYY [integer integer]]
:: output - [Wwks, ][Dday, ][Hhrs, ][Mmin, ]S.Csec
:: returns time lapsed from %TIME% [%DATE%] to present in TS_ or if given
:: part/whole will return estimate of time remaining to completion in TR_.
:: raw time and day info for given(1) and current(2) are also available.
:: TM1,TM2=# of centiseconds since last midnight.
:: DY1,DY2=# of days since noon on January 1, 4713 BCE (Julian calendar).
:: usage:
:: SET "startTime=%TIME%" or "startTime=%TIME% %DATE%" before event then:
:: CALL :timeSince %startTime%                      - for current elapsed time.
:: CALL :timeSince %startTime% %partCnt% %wholeCnt% - to estimate by count.
:: CALL :timeSince %startTime% %partKb%  %wholeKb%  - to estimate by size.
:: v0.2 2018/08/06
::
SETLOCAL EnableDelayedExpansion
SET res=
SET inp=%*
SET var=TS_
SET/A p=t=t1=tm1=t2=tm2=d=d1=dy1=d2=dy2=y=z=0
IF "!inp:~1,1!"==":" SET inp=0!inp!
FOR %%@ IN ("!inp!" "!TIME: =0! !DATE!") DO (SET/A p+=1
	FOR /F "tokens=1-10 delims=:./ " %%A IN ("%%~@")DO IF "%%D" NEQ "" (
		SET/A"tm!p!=t!p!=(((1%%A*60)+1%%B)*60+1%%C)*100+1%%D-36610100"
		IF "%%H" NEQ "" (SET/A mm=100%%F%%100,dd=100%%G%%100,yy=10000%%H%%10000
			SET/A"dy!p!=d!p!=!dd!-32075+1461*(!yy!+4800+(!mm!-14)/12)/4+367*(!mm!-2-(!mm!-14)/12*12)/12-3*((!yy!+4900+(!mm!-14)/12)/100)/4")
			IF "%%J" NEQ "" SET/A y=%%I,z=%%J))
IF !t1! GEQ !t2! SET/A t2+=8640000,d2-=1
IF !d1! EQU 0 SET d2=0
SET/A t=t2-t1,d=d2-d1
FOR %%A IN (1 2 3) DO IF !z! GTR 9999999 SET/A y/=10,z/=10
IF !y! LSS 1 SET y=1
IF !z! NEQ 0 SET/A "t=(z-y)*100/y*((d*8640000+t)/100),d=t/8640000,t=t%%8640000"^& SET var=TR_
SET/A w=d/7,d=d%%7,h=t%%8640000/360000,m=t%%360000/6000,s=t%%6000/100,c=t%%100
IF !c! LEQ 9 SET c=0!c!
IF !w! GTR 0 SET "res=!w!wks, "
IF !d! GTR 0 SET "res=!res!!d!day, "
IF !h! GTR 0 SET "res=!res!!h!hrs, "
IF !m! GTR 0 SET "res=!res!!m!min, "
SET res=!res!!s!.!c!sec
(	ENDLOCAL
	SET "%var%=%res%"
	REM SET "tm1=%tm1%"
	REM SET "tm2=%tm2%"
	REM SET "dy1=%dy1%"
	REM SET "dy2=%dy2%"
)
EXIT /B 0

:wait centiseconds
:: high CPU load, only suitable for very short delays
FOR /F "tokens=1-4 delims=:." %%w IN ("%TIME: =0%") DO SET/A "waitTime=(((1%%w*60)+1%%x)*60+1%%y)*100+1%%z-36610100"
IF NOT DEFINED waitStart SET/A waitStart=waitEnd=waitTime,waitEnd+=%~1
IF %waitTime% LSS %waitStart% SET /A waitTime+=8640000
IF %waitTime% LSS %waitEnd% GOTO :wait
SET waitStart=
EXIT /B 0

:grabKey "" exitKey keyFile
ECHO(^%key% >%3
SET "key="
FOR /F "skip=1 delims=" %%# IN ('replace ? . /u /w') DO IF NOT DEFINED key SET "key=%%#"
IF NOT EXIST %3 EXIT
IF /I "%key%" NEQ "%~2" GOTO :grabKey
DEL /F /Q /A %3 >NUL 2>&1
EXIT

:doErr
ECHO(%TIME: =0% %DATE% >>"%~f0.Error.txt"
SET>>"%~f0.Error.txt"
ECHO(_______________________________________________________________________________>>"%~f0.Error.txt"
EXIT /B 0

:mazing_init
IF NOT DEFINED TEMP SET "TEMP=%TMP%"
IF NOT DEFINED TEMP SET "TEMP=%~dp0"
SET "PATH=%TEMP%;!PATH:%TEMP%;=!"
(	FOR /F "tokens=1 delims==" %%A IN ('SET 2^>NUL') DO SET "%%A="
	SET "TEMP=%TEMP%"         'assure existence of TEMP, use script folder if needed
	SET "PATH=%PATH%"         'clean environment reserving only required variables
)
:: Start User Variables
SET "totalTime=%TIME% %DATE%" 'script startTime for :timeSince function
SET "maxCols=78"              'maximum # of columns allowed in menu
SET "minCols=12"              'mainmum # of columns allowed in menu
SET "maxRows=38"              'maximum # of rows allowed in menu
SET "minRows=5"               'minimum # of rows allowed in menu
SET "maxSize=8186"            'maximum # of characters allowed in maze
SET "minSize=512"             'minimum # of characters allowed in maze
SET "menuKeys=AZ[]123H"       'up,down,left,right,continue+resume,quit+restart,abort+exit,help
SET "exitKey=M"               'which keypress will interrupt the script/show menu
SET "logFile=%~dpn0.mazes.txt"  'output text file for both maze and solution, no file if undefined
SET "keyFile=%TEMP%\mazing.key" 'filename used by :grabKey subroutine
SET "cfgFile=%Temp%\Mazing.cfg" 'file for saving mzOp# values
SET "iniFile=%TEMP%\Mazing.ini" 'file for setting user variables
:: default character strings for random generation
SET "walls=#%%&08@" 'chosen for being large and bold, character repeated to be selected more often
SET "halls= "                 'space is the only hall character that seems to work well
SET "crumbs=."         'chosen for being small and centered
SET "players=S"          'chosen because the look like a player or entrance
SET "goals=?$X"           'chosen because they look like an exit or goal
SET "colors=0123456789ABCDEF" 'hex numbers for foreground and/or background selection
SET "fColors=1234569ABCDE"    'hex numbers for foreground selection
SET "bColors=078F"            'hex numbers for background selection
:: list of bad color combos to replace with good color combo
SET "badClrs=68 86 8C C8 8D D8 EF FE"
SET "goodClr=80"
:: End User Variables

:: read ini file and command line options
IF EXIST "!iniFile!" FOR /F "usebackq tokens=*" %%A IN ("!iniFile!") DO SET "%%A"
FOR %%A IN (%*) DO FOR /F "tokens=1* delims=:=" %%B IN ("%%~A") DO SET "%%B=%%C"

:: set code page, start keyGrabber, set constants
MODE CON CP SELECT=437 >NUL%=     'codepage required for box characters, if no rebuild, ascii should be OK =%
REM.>"%keyFile%"%=                'pre-generate keyFile if grabKey is too slow =%
START "" /B CMD /C ^""%~f0" grabKey %exitKey% "%keyFile%" 2^>NUL ^>NUL^"
SET "display=-1"                  '0=no display/log only, -1=display, 1=display+color using BG.EXE
SET "version=0.1"                 'current version # for Mazing.cmd
SET "EKO=<NUL SET/P="             'display to screen without CR/LF, faster than the executables
SET "clear=CLS"                   'will be replaced by BG or CursorPos if available
SET "delay=0"                     'delay in centiseconds to slow process for viewing
SET "title0=Mazing Menu"          'string showing current operation in TITLE
SET "title1='%exitKey%' for Menu" 'notice displayed at the end of TITLE
:: read character list from bottom of script
FOR /F %%A IN ('TYPE "%~f0"^|FIND "" /v /c') DO SET /A t1=%%A-1
FOR /F "usebackq skip=%t1%" %%A IN ("%~f0") DO SET "chars=%%A"

:: discover the length of each character string
FOR %%A IN (wall hall crumb player goal color fColor bColor char) DO ( SET %%ACnt=0& SET "t1=!%%As!0"
	FOR %%B IN (128 64 32 16 8 4 2 1) DO IF "!t1:~%%B,1!" NEQ "" SET /A %%ACnt+=%%B & SET "t1=!t1:~%%B!")
SET /A charCnt-=1%=               'adjust for zero-indexing =%

:: box characters for rebuild
::
::       2
::   6 C E C A
::   3   3   3       +1
:: 4 7 C F C B 8   +8 0 +4
::   3   3   3       +2
::   5 C D C 9
::       1
::
::  "box?=123456789ABCDEF" no zero, because all walls are connected
SET "box0=|||-\/+-/\+-+++"
SET "box1=|||-\/+-/\+-+++"
SET "box2=ٿ"
SET "box3=ͼ"
SET "box4=Ľ"
SET "box5=;"
SET "box6="
SET "box7="
SET "box8="
SET "box9="
SET "box10="
SET "box11=!chars:~20,1!ٿ"
SET "box12=|\/+!chars:~20,1!/\+-+++"
:: box characters for menu
:: @=3 {=6 [=5 }=9 ]=A $=C
::  "mBox?=012345"
SET "mBox_=@{[}]$"
SET "mBox0=|\//\-"
SET "mBox1=ٿ"
SET "mBox2=ɼ"
SET "mBox3=ֽ"
SET "mBox4=վ"
SET "mBox5="
SET "mBox6="
SET "mBox7="
:: main menu, %version%=3 chars, @ { } [ ] $ are reserved
SET mzMenu=  [$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$]  ^
  @                                                      @  ^
  @  Mazing.cmd - WinNT Maze Generator and Solver  v%version%  @  ^
  @                                                      @  ^
  @ [$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$] @  ^
  @ @                                                  @ @  ^
  @ @   Maze Generation Algorithm       :0:  @ @  ^
  @ @   Maze Solving Algorithm          :1:  @ @  ^
  @ @   Select Entrance/Exit Point      :2:  @ @  ^
  @ @   Width in Vertical Columns       :3:  @ @  ^
  @ @   Height in Horizontal Rows       :4:  @ @  ^
  @ @   Stack Size/Type/Wall Setting    :5:  @ @  ^
  @ @   Node Selection %%Bias            :6:  @ @  ^
  @ @   Node Direction %%Bias            :7:  @ @  ^
  @ @   Box Convert                     :8:  @ @  ^
  @ @   Program Delay                   :9:  @ @  ^
  @ @   Wall Character                  :10:  @ @  ^
  @ @   Crumb/Trail Character           :11:  @ @  ^
  @ @   Start/Entrance Character        :12:  @ @  ^
  @ @   Finish/Exit Character           :13:  @ @  ^
  @ @   Background Color                :14:  @ @  ^
  @ @   Foreground Color                :15:  @ @  ^
  @ @   Console Display Type            :16:  @ @  ^
  @ @                                                  @ @  ^
  @ @  :A:=  :B:=   :C:=Resume/Restart/Exit  :D:=Help  @ @  ^
  @ {$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$} @  ^
  @                                                      @  ^
  @                                                      @  ^
  @                                                      @  ^
  {$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$} 

SET "hex=0123456789ABCDEF" 'hexadecimal table for COLOR command
SET "mrk=>> "              'menu selection marker
:: mWide,mHigh=menu WxH, opCnt=#of menu items-1, opSize=length of option text, orgPos=default position in menu, mt#=mazingTemp variables
:: opSize=string length of menu items to be injected into %mzMenu%, orgPos=position in %mzMenu% for 1st menu item
:: mzOp#P#=menu position for option titles, for retitling options
SET /A mWide=60,mHigh=32,mSize=mWide*mHigh,opCnt=16,opSize=13,orgPos=400,mt0=mt1=mt2=mt3=mt4=-1,mt5=mCh=0
SET /A mzOp5P1=mzOp5P2=668,mzOp5P2+=28,mzOp6P1=mzOp6P2=728,mzOp6P2+=28,mzOp8P1=mzOp8P2=860,mzOp8P2+=15,msgP1=msgP2=1625,msgP2+=50
:: load arrays for menu display, user variables, and maximum size for menuOps
FOR %%A IN ("Random" "Backtracker" "Hunt & Kill" "Growing Tree" "Itr. Division" "Eller's Algor") DO SET "mzOp0[!mt0!]=%%~A" & SET /A mt0+=1
FOR %%A IN ("Random" "Don't Solve" "Wall Follow" "Dead Filler" "Path Finder") DO SET "mzOp1[!mt1!]=%%~A" & SET /A mt1+=1
FOR %%A IN (Random Inside Border Corner) DO FOR %%B IN (Random Inside Border Corner) DO SET "mzOp2[!mt2!]=%%B/%%A" & SET /A mt2+=1
FOR %%A IN ("Stack Size/Type/Wall Setting" "Unused by Backtracker       " "Stack Type LIFO/FIFO        " "Stack Type + List Size      " "Placement of Wall Openings  " "Unused by Eller's Algorithm ") DO SET "mzOp5[!mt3!]=%%~A" & SET /A mt3+=1
FOR %%A IN ("Node Selection %%Bias        " "Unused by Backtracker       " "Node Selection %%Bias        " "Node Selection %%Bias        " "Order of Iterative Stack    " "Unused by Eller's Algorithm ") DO SET "mzOp6[!mt4!]=%%~A" & SET /A mt4+=1
FOR %%A IN (4 3 14 !maxCols! !maxRows! 24 101 100 14 101 !charCnt! !charCnt! !charCnt! !charCnt! 16 16 1) DO SET /A opMax!mt5!=%%A,mt5+=1
FOR /L %%A IN (0,1,%opCnt%) DO SET mzOp%%A=-1& SET "mView%%A=Random       "%= 'mzOp#=menu values, mView#=menu option display, -1=Random =%
SET "mView2=Random/Random"
SET "mView9=None         "
SET "mView16=16 Color     "
SET "mzOp9=0"
:: messages shown at the bottom of the menu for each option chosen
SET "mMsg0=generate perfect maze with 1 path for any 2 points"
SET "mMsg1= then find the unique solution for generated maze "
SET "mMsg2=entrance is random,exit is furthest possible point"
SET "mMsg3=  maze width=# of columns, character width=W*2+1  "
SET "mMsg4=  maze height=# of rows, character height=H*2+1   "
SET "mMsg5= stack/wall setting, different for each generator "
SET "mMsg6=    0=always random, 50=even, 100=always stack    "
SET "mMsg7=  0=most vertical, 50=even, 100=most horizontal   "
SET "mMsg8= convert single character maze to box characters  "
SET "mMsg9=   # of centisecond to delay for better viewing   "
SET "mMsg10=        single character to use for walls         "
SET "mMsg11=     single character to use for crumbs/trail     "
SET "mMsg12=    single character to use for start/entrance    "
SET "mMsg13=     single character to use for finish/exit      "
SET "mMsg14=   hexadecimal background color, remains static   "
SET "mMsg15=   hexadecimal foreground color, remains static   "
SET "mMsg16=   native 16 color, BG.EXE color, or no display   "
:: arrays to allow multiple messages for any option, keyed to mzOp#
SET "mMsg5[-1]=%mMsg5%"
SET "mMsg5[0]=  setting is not used by Depth-First Backtracker  "
SET "mMsg5[1]=    stack direction setting 0=FIFO, not 0=LIFO    "
SET "mMsg5[2]=   list size+direction 0=all, >0=LIFO, <0=FIFO    "
SET "mMsg5[3]=wall open 0=NW, 1=SE, 2=NW/SE, 3=center, 4+=random"
SET "mMsg5[4]=     setting is not used by Eller's Algorithm     "
SET "mMsg6[-1]=%mMsg6%"
SET "mMsg6[0]=%mMsg5[0]%"
SET "mMsg6[1]=%mMsg6%"
SET "mMsg6[2]=%mMsg6%"
SET "mMsg6[3]=    stack order 0-7=static pre-sets, 8+=random    "
SET "mMsg6[4]=%mMsg5[4]%"

:: config file
IF EXIST "!cfgFile!" ( FOR /F "usebackq skip=1 tokens=*" %%A IN ("!cfgFile!") DO SET "%%A"
	FOR /L %%A IN (0,1,%opCnt%) DO SET mCh=%%A& CALL :mazing_menu_cfg 0
	SET "tMenu=" & SET mCh=0
)

:: use CursorPos or BG if found in path
CursorPos.exe >NUL 2>&1
IF %ERRORLEVEL% NEQ 9009 SET clear=CursorPos.exe 0 0
BG.EXE >NUL 2>&1
IF %ERRORLEVEL% NEQ 9009 SET clear=BG.EXE Locate 0 0& BG.EXE Cursor 0
EXIT /B 0

:makeBG
IF EXIST "%TEMP%\BG.EXE" EXIT /B 1
TITLE extracting BG.EXE, please wait...
DEL /A /F /Q "%TEMP%\BG.TMP" "%TEMP%\BG.EX_" >NUL 2>&1
(
ECHO(TVNDRgAAAABhDgAAAAAAACwAAAAAAAAAAwEBAAEAAAAAAAAAQwAAAAEAAxIAJgAA
ECHO(AAAAAAAAN025dSAAQkcuRVhFAC0aSdkWDgAmW4CAjQIgBmBt1ADFYwAkIgBwAADf
ECHO(f+m6dTp8PWkUDlnQ+A6QSYqkckDv5dwYCmNqhWHumj8Y3avpflOcfT/91G6/bDg8
ECHO(ZBh7oDxn73fO24cSyBkmzGs4u5HzssGQQTXOh8wDn1qSIB4EADMygFYIAPtw3vtv
ECHO(31SuvZmLAMPB5N/AW+wdZ/QKXe8dhFhA2W27NRmhrLPtdWt2rrb2td3XqrdlVm1Z
ECHO(3GJGbSxRKyTCVWkarWLI4HQESH4C4/8AAgAABACMisgAvxvtfbf3zGz0xAiT4ldE
ECHO(qUEwAguruIKg/y5yAckYn/FQAmIIcnkDBIU+MNtGhBgXpqrQ9wLVKb5vfX3xjw1n
ECHO(Lbe2Fjo0c8flaxoscNbY4uIh5GFrxoK4jW6Wi/+ZFbOYk4OFaUXFmkp81qXMY8aY
ECHO(DB1Oz3c044oZqgi+h5UMKlRutC5TMyT9nioZE8kE8mbxWmIHXgep2vs985D5D2Vb
ECHO(whU9y3s68slxKMGvO7JQdLlV/BBcsMMtgNNVqy51Ap+IW8bhWtamGrEM6QsMNr5v
ECHO(wYNful7C1my5dHTJQ9VS4/4fIRuXo6XZxxWUFiCii/XrkhfzIusjv0DmdGI1y+ww
ECHO(xXGEsPhLrMpEBfQKympNhA8bWf4I/PbqVRysaZwztAML9iEHCRks6nucI0M4ZC9L
ECHO(IILh/O4lJPB4mJXJyGvIW0kAprRK4r2trMkKz4he9lLHEriPSdHeQHLCe4MJP0jI
ECHO(wKSnmFVS/Ij4Dj6nR7wCp6UwHpXB3OSNw28ymyLKxQgWz60l5VVnD+akzmatPc6W
ECHO(w44VemWpotPUFPLJmHS5J9tsHCLk56NOdF3H4g1oLQTX6ossEWsK5pl4SD/G44xN
ECHO(bYtd7vlafq9H2P2hdWEwbsSKRcNFZ5CURLZKa3o8kGy2WZjn4q1j4zYEsvt9gSYO
ECHO(Be7xfotZzbhSB6bGUHIr972hoWCsBI32KgnuQH5qpmGWlNWO02o+WlrlkJSiH9eP
ECHO(aV+mJA7gQdI6r3L013MnOaBL9S5SBjke0voKlByA4Nd5B0DBi0hSMFCRaMUjjM9d
ECHO(gTjdl5KOHVRj2c+OMyEXUKomW+nqIYIRSl4YF8zMyqHVIYpk7ggau/lPl5DZD3dT
ECHO(nviyDZ8IgomDVEKng1BprojzgYr0EZmPvBFNZ3JD2WeMsreCMJUUwcLZ+8cmnSxi
ECHO(V9n86UCGoin4t8ynfQbqtCt9FW8+T0AybVJCGaic0/A74R30lKtWCfqH+3ZIncPc
ECHO(09tcXe0hB5oDnbDAPM6oNyCEij/mvEdoUJJ9hm6Vpf++Sc+jv7bWf7/9rfkPemNt
ECHO(LV7B6Ns+PeaqZGdlPg7rVaO3BKHjk1FVPhHE6E7RVHMeuIjdIXORkSa4Fwv4Cc+f
ECHO(3jLn03/tdTe5nNry5GSI/bJ9JOUf/y/Frx/ff3fa+7IeS8bNkCBff2NnJEqO+T8U
ECHO(QXe16B9i0BMFsNcelFqRr5Ds9nw011hWAyXvCFU9hbV5FQJg+xJob+rmqLJDWaEE
ECHO(FyqWWaIv86MTLGRXCVuu3Dx9BNfYJJagyNse72feiKBX5np0aMk8knQM9xIIRvdp
ECHO(fa7WU1y41pD6rrobgXx3yg/zBa9z+vLmFxeParz8Dx0ATvAEmMo7j+gt2iAK7MMF
ECHO(BU5gNCHFRMi0ilTzQZISdnFuxpcVp788+qGf+GNaGI8Zxxs1Ry7an8kSxXk6ikJ5
ECHO(RKLJJ1jHnamHHZMq7HZyBbKPqGnLPN4WYvqOzY5wmsOlqArrm1fN4cmYGTdDmXfI
ECHO(ZkRuxFbLUU1bkfw/dHXCmW+tXdkJpwfTq7vLwnper2ri2bqpVm/qkVfCE+w/S6yQ
ECHO(hXjxMzn0VnnXFQlkoA3OoaGzZBkfgywCsDHAVsBeB1AFAGMIphQwNwa/ah5v17cC
ECHO(bH89z8NTGPHGFRppWkZ5ELmmgnzpJDUh9Ska9aTwOebOQKkuLb4pzne77IUeAC++
ECHO(g0cS33R2bVPqZGGmSjxOCWLqG5BRm0HK6c3fcuKPxzGzoEv6JIpTOmBd1XpD5yU8
ECHO(NI+eiOZpCmyNgnDTlf7kkhna8RrDg1SkpbHyxWF4TOdNYZyz9ARQxvj9p8CwVZDR
ECHO(HTYDnX4lpn7icOaRJu69RgV7S/GcZu6aQCsfKkQMEEy+JsTEM5xfmMD2+nopCCzB
ECHO(hYcNvkKoOjY2n/QzsMV4UlMm+etq8AxjNAzba2RVV2Tny9a/pk617ST+GA/d3m2T
ECHO(9TEBfiRvHGauAr/NVowKRun2xDDocR9KVVsY4UzBFjNPqZehrVkD1cIgFv+nN3q/
ECHO(Ysf6N8Fk+l+1mafrVD5UUMwFp6QKPF30m9InCKYANxcFl5qxY2hbBjm6Hrgr7qWs
ECHO(+udJw7hKj6nyguoFF7FLz7xaOMhFAMYpT2G3FavRrJfAMoJzLc58e/F8wjdGYKLy
ECHO(kEfsIllSvVtF6AP/ff8vteov0CM/w6sB2GB2g9pd97rKLx5XVCZlEvtGLrhTeABg
ECHO(1goJesnJOjp4Ub4g7+pVHso2tO711o/ZGNV7X5GbPYf9s4R1nn2MX64SgSa5ZMpL
ECHO(hmUq8F4OPs8ol+p1lMMTxfLK0+Zglwg7ileyAe5F39IhIdSsukUzhOxo+4GvSZ3y
ECHO(iuF73SjIAxivclFuRVRsvJb78PvVO0gFJETid0KjNA7kRPQ/Jwk9POUkeeT9e38g
ECHO(tnv2sbNo265t/ugWgG8UoqDHCcX20h4PFPWk+FePckNRTr77Kt4lNlK/epRtMG0w
ECHO(4c8SgP13vDT4CD/1mAhFoZec5qjUKF2JRYYCueVeQ9dfIRLV9Yw84x8B6k9C6/Ej
ECHO(UdGyEZ72R3lMpKtN9Rbx2BHKqLCVkVGhgfAB8OP0eQDfmJBkUfcnfYkm/6AFEJoi
ECHO(lGJ5NXEZpVmpJ3YZfiNe7Q4ZVyJeUqVsGwnEQMFKE9clGLPz5ODB8T6YM93FKtOl
ECHO(6rLuKn2UR/rpXBSLjEDpQxOM3K1JwVDekaZacxgZAFWR/dbVDV2g+LyR1LJS1DZb
ECHO(+MTxk+QR2GXXx/+7lIFWkuwbjAVZlW04jX8Ta0yZns/TVZzNeBN93bomEEKyRHPb
ECHO(xkJOhITCGu2a/+MP6Zy4xmzmg9W94oEa+HyGSGEYDTwUOqwvGnmM8L308I2m7Suo
ECHO(Y0K9X7RkJ7C+O7ftzQ4ykqaPeqc6s31ydjWU+kOkK4yov8J1I+qWFenvub3x8yr6
ECHO(jjyosd5gSu6+8ptWe33AqwSAaK8UDEbmh4jhuJIrNgcEyGstEDZR5In1lo6MdWBW
ECHO(14u1x/pb1ExbgdrLuKvk1mkrVdpCq5oRufSPLJmdtikKNc1IYmCv6zztTqj/ug52
ECHO(ZHQjRFAAASW6kcyUArqaEMNcPg0xJEd1WOmNTz6/n0UqWFNXvnsj1UVaOYe08Or1
ECHO(3ooW1ZgdJOnEIbMgkskf3xAka8JA+saPZUrAyMZ7PyQj+v1Du179WEegX2i6X5Xr
ECHO(w+vjBRSqbzVb8QaXD9R0ayqvgB85Il6BonyC79pZ9ALBgQ+N95ZGkF3SHkvkzsIb
ECHO(LA2LjUKGqhXC/OewPZkSbRw97SzNS2F7v112T/mic7/O3JK16R/YV3AtbqhsyapV
ECHO(yrn253YAX++bxPMQKlR0bZIev786llXHWYfyE33wtY4LnJWzh6+eP7zzPP2/jgaA
ECHO(m0SofjGkopUgKJ2wDkf46x/8v6qHUCMXXxgpUSCrmeW6iGi1YFPVzDNDJQ+SipB/
ECHO(2o84TOLgDuK/ciWd0u4l66A9sRNY9fYrLOPTNNj0uY/S4BNOT+QMmTJdvIfZivYI
ECHO(hM8YzLkV6Ja65SxBwt8gt15bWDji3EFvx9oeaDux9iPtY/GOsz3GdpDdIWDHvB74
ECHO(OqPrh9YdW3JorMOwjrA6o+qOqXJQhGM7PEQsRywQclg79r+sIHHBczlMQHnh7v8i
ECHO(IFnhSNKMwQwu2CqBS0YxcYyY8ZbAZiMYjUZxOLk5upg3jLF3IeRsETCcoG4CxAmw
ECHO(JuSbyXPaum8R3PznVKtQXC5snN/Z0tr72XBW5FggeXs+8hiQ5O4ucErhsdJ8GY0U
ECHO(s5z6+DTH3ZnkOTVJMxazsngsy5p5Rpl2NJZVzV+4EjLNGCs205dlxfc90tp34Sdy
ECHO(VtzLcWWMqwz8WDUobKofYqNrQTAGoAoA7GBZmGx+Ly79kV5VGTuU3MgXAIZVBV94
ECHO(x+IbrFgCVmLLuDZXVwr+/va4Wry4urh1F2uT4i8mWKFhl1KwuH9aF3OUFXaDQiyB
ECHO(Lhjc1O8II4EeWd1HyTOfvC6fAwAEgSCB14JOzOrgUhg+4CzB4wdyCy4gyIUB+weR
ECHO(dT+WkytEAVX4q4qyAtgKARgDZDg4W9D8Rrsm6cl3msjyE9uTe4uhWyNri4B5RLnR
ECHO(iLGGWAyz20obDGizu4xwnKYMm19/bd7HDowdtzENW5o1hbcPdxjiS9l1/zQqldXL
ECHO(xCSVsJl/MnlxZz8R57KZrlQxPCazZ0wygWNAkdC9rbmdOMPSRTqarfCmrVu0tAog
ECHO(oW1tOLOQ0sr5cxZnA0zumXsSsWJ/zaBqO39f4zb1LOX5klvtSALZlpxw+8TJ/unF
ECHO(qXOGwg6icGoI8WwjsZGoJdHmMDQWSpuQSBrwHc2IdAW3tWq4W03pa3Bxbuubn45E
ECHO(KVE7CpoZ9/y1YCaNj9SWrJmzaZQtI5YLubVlOLnxnFMrcqZ+tKA8rZh3kxzb9hyE
ECHO(hMrMR+INmG6TR3RvCZFzyUsfOdC3zL7yeFZ5O8dyfJTdQWPPJirlOYObmiEcsJDZ
ECHO(YTiMuTOtWL29/Q+lBJkg2lvs7UbO/Oj0ae/e7voL4vjZ7JIdfHFtvbVbvsxqfLEd
ECHO(XFsdCQ0R4ShtzND9mhgFrTJeNpElcZGNiZ+OVsIyL4A9
)>"%TEMP%\BG.TMP"
CERTUTIL /DECODE /F "%TEMP%\BG.TMP" "%TEMP%\BG.EX_" >NUL
EXPAND -R "%TEMP%\BG.EX_" -F:* "%TEMP%" >NUL
DEL /F /Q /A "%TEMP%\BG.TMP" "%TEMP%\BG.EX_" >NUL
EXIT /B 0
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
BG v3.9 se (unicode)

BG.EXE is a tool for print text color in cmd.exe. It accepts regular
expressions for print ascii characters. It also have useful functions.

Notice: This version fixes a problem happened on windows 7 sp1 with the function font.

::::: Parameters :::::

Kbd
::Wait until you press any key, and returns the ascii code.
::If an extended key was pressed, returns the second code plus 256.
::Example. If you pressed the up arrow (224, 72) returns 327 (72+256).

LastKbd
::If any key was pressed, returns the ascii code, else return 0.
::If the key pressed is extended, it returns the second code plus 256.
::Example. If you pressed the up arrow (224, 72) returns 327 (72+256).

Mouse
::Wait until you press primary button into console area.
::When you press a valid button of mouse print: rowNumber colNumber (both 0 index based).
::Also, is returned a number, where you can get the same info:
::Set /A "mouseRow=%ErrorLevel% >> 0x10 , mouseCol=%ErrorLevel% & 0xFFFF"

DateTime
::Print eight numbers, separated by a space, that corresponds to the current:
::DayOfWeek Year Month Day Hour Minute Second Milliseconds
::
::DayOfWeek value is from 0 to 6. 0 is Sunday, 1 is Monday, ..., 6 is Saturday
::Year value is from 1601 to 30827
::Month value is from 1 to 12
::Day value is from 1 to 31
::Hour value is from 0 to 23
::Minute value is from 0 to 59
::Second value is from 0 to 59
::Milliseconds value is from 0 to 999

Cursor 0 | 1 | 25 | 50 | 100
::hide or show cursor of keyboard or change the size.
::0 hide
::1 show
::25 small size
::50 medium size
::100 large size

Font 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9
::change the size of font to terminal font according this next table:
:: index    size
:: 0        4x6
:: 1        6x8
:: 2        8x8
:: 3        16x8
:: 4        5x12
:: 5        7x12
:: 6        8x12
:: 7        16x12
:: 8        12x16
:: 9        10x18

Sleep milliseconds
::do a wait or sleep in milliseconds. milliseconds should be greater than 0.

Locate row column
::locate the position of cursor in row and column specified (both 0 index based).
::if row is less than 0, is interpreted as 0.
::if row is greater than the max row (in the console screen buffer), is interpreted as the max row.
::if column is less than 0, is interpreted as 0.
::if column is greater than the max row (in the console screen buffer), is interpreted as the max row.

Print text
::print text without new line to end. The text can use oem escape sequences.
::please see the regular expressions in down.

Print color text ...
::print text in HEXADECIMAL color without new line to end. You can repeat arguments.
::the text recognize the next oem escape sequences:
\n print a newline
\\ print \
\Number  HEXADECIMAL ascii number code from 00 to FF.
Example: \01 is face character
Example: \41 is A
Note 2: You can repeat arguments.
Example: BG.EXE Print A "Text in color A" B "Text in color B"

FCPrint row column [color text ...]
::begin printing of colored text in the row and col (zero index based). The column is fixed.
::row and column can be outside the console screen buffer limits.

Play soundfile.wav [ntimes]
::play soundfile.wav from begin to the end
::if ntimes is specified the play is repeated the number of times specified.
::(this function is different from the utility sound.exe where the second parameter correspond to seconds.)

::::: Colours :::::
[background]foreground
colours values is a one hexadecimal (0123456789ABCDEF) digit for each concept.

0 = Black
1 = Blue
2 = Green
3 = Aqua
4 = Red
5 = Purple
6 = Yellow
7 = White
8 = Gray
9 = Light Blue
A = Light Green
B = Light Aqua
C = Light Red
D = Light Purple
E = Light Yellow
F = Bright White

:: characters must remain the last line of the script - DO NOT REMOVE!
"#$%&'()+,-./0123456789:<>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~
